#include <windows.h>
#include "resource.h"
#include "Main.h"
#include "AdapterGUI.h"
#include "Functions.h"

/* Function Declarations for internal use */
LRESULT CALLBACK AdapterGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

/* Variables */
long lAdapters;

/* The main routine */
BOOL MainAdapterGUI(HINSTANCE hInstance, HWND hWndOwner)
{
	long lAdapterIdx;

	/* Find all available adapters */
	lAdapters = FindDevices();
	if (lAdapters < 1)
	{
		MessageBox(hWndOwner, "No network card detected.", "Error", MB_ICONERROR);
		return false;
	}

	/* Show the adapter dialog */
	lAdapterIdx = DialogBox(hInstance, MAKEINTRESOURCE(IDD_ADAPTERGUI), hWndOwner, (DLGPROC)AdapterGUIProc);
	if (lAdapterIdx < 0)
		return false;

	/* Set adapter to be used, and return success */
	SetActiveDevice(lAdapterIdx);
	return true;
}

/* Window message handler */
LRESULT CALLBACK AdapterGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	long lLoop;
	HWND hWnd_lstAdapter;

	/* Init variables */
	hWnd_lstAdapter = GetDlgItem(hDlg, IDC_lstAdapter);

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			for (lLoop = 0; lLoop < lAdapters; lLoop++)
			{
				SendMessage(hWnd_lstAdapter, LB_ADDSTRING, 0, (LPARAM)GetDeviceName(lLoop));
			}
			SendMessage(hWnd_lstAdapter, LB_SETCURSEL, 0, 0);
			SetFocus(hWnd_lstAdapter);
			SetForegroundWindow(hDlg);
			return true;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_lstAdapter:
					if (HIWORD(wParam) == LBN_DBLCLK)
						EndDialog(hDlg, SendMessage(hWnd_lstAdapter, LB_GETCURSEL, 0, 0));
					break;
				case IDC_cmdSelect:
					EndDialog(hDlg, SendMessage(hWnd_lstAdapter, LB_GETCURSEL, 0, 0));
					return true;
				case IDC_cmdCancel:
					EndDialog(hDlg, -1);
					return true;
			}
			break;
	}

	return false;
}
