#include <windows.h>
#include <richedit.h>
#include <shlobj.h>
#include "resource.h"
#include "Main.h"
#include "FirstTimeGUI.h"

/* Function Declarations for internal use */
LRESULT CALLBACK FirstTimeGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

/* Constants */
static char strBanner[] = "\nAO Mission Display v1.4\nBy Muzzleflash (http://muzzleflash.da.ru)";
static long lStrBannerTitleS = 1;	// strBanner selection for title
static long lStrBannerTitleE = lStrBannerTitleS + 23;
static char strInfo[] = "Welcome! Please take the time to read this info before using. It will only appear once.\n\
\n\
This tool will allow you to view the location, cash and item rewards of all missions currently being provided by the mission terminal, in a single up-front window. I wrote this tool after getting sore wrists from shift-clicking each mission and scrolling to the bottom to determine the reward. Now with this tool you can see immediately what each mission is offering, allowing you to just sit back and keep hitting the 'Request Mission' button until you get the one you want.\n\
\n\
There are a few things you need to be aware of:\n\
You must first download and install WinPcap from http://netgroup-serv.polito.it/winpcap/ to use this utility. Note that only the latest v2.3beta supports WinXP.\n\
Your connection to the internet must be via an Ethernet card (a requirement of WinPcap). It will not work on dialup modems.\n\
You must set AO to 'Run in Window' mode. The AOMissionDisplay window will not show if you run the game in full screen mode.";
static long lStrInfoNbrS = 614;		// strInfo selection for numbering
static long lStrInfoNbrE = -1;
static long lStrInfoWarnS = 864;	// strInfo selection for bold warning
static long lStrInfoWarnE = lStrInfoWarnS + 34;

/* Variables */

/* The main routine */
BOOL MainFirstTimeGUI(HINSTANCE hInstance, HWND hWndOwner)
{
	/* Show the first time dialog */
	if (!DialogBox(hInstance, MAKEINTRESOURCE(IDD_FIRSTTIMEGUI), hWndOwner, (DLGPROC)FirstTimeGUIProc))
		return false;

	/* Success */
	return true;
}

/* Window message handler */
LRESULT CALLBACK FirstTimeGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hWnd_txtBanner;
	HWND hWnd_txtInfo;
	CHARFORMAT2 udtCF;
	PARAFORMAT2 udtPF;
	ENLINK *udtENLink;

	/* Init variables */
	hWnd_txtBanner = GetDlgItem(hDlg, IDC_txtBanner);
	hWnd_txtInfo = GetDlgItem(hDlg, IDC_txtInfo);

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			/* Set banner auto www links */
			SendMessage(hWnd_txtBanner, EM_AUTOURLDETECT, true, 0);
			SendMessage(hWnd_txtBanner, EM_SETEVENTMASK, 0, ENM_LINK);
			
			/* Set banner text */
			SendMessage(hWnd_txtBanner, WM_SETTEXT, 0, (LPARAM)strBanner);

			/* Set banner title to bold/underline */
			SendMessage(hWnd_txtBanner, EM_SETSEL, lStrBannerTitleS, lStrBannerTitleE);
			udtCF.cbSize = sizeof(udtCF);
			udtCF.dwMask = CFM_BOLD | CFM_UNDERLINE;
			udtCF.dwEffects = CFE_BOLD | CFM_UNDERLINE;
			SendMessage(hWnd_txtBanner, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&udtCF);

			/* Set banner title spacing */
			udtPF.cbSize = sizeof(udtPF);
			udtPF.dwMask = PFM_LINESPACING;
			udtPF.bLineSpacingRule = 5;
			udtPF.dyLineSpacing = 25;
			SendMessage(hWnd_txtBanner, EM_SETPARAFORMAT, 0, (LPARAM)&udtPF);

			/* Set info box auto www links */
			SendMessage(hWnd_txtInfo, EM_AUTOURLDETECT, true, 0);
			SendMessage(hWnd_txtInfo, EM_SETEVENTMASK, 0, ENM_LINK);
			
			/* Set info box text */
			SendMessage(hWnd_txtInfo, WM_SETTEXT, 0, (LPARAM)strInfo);
			
			/* Set info box paragraph numbering */
			SendMessage(hWnd_txtInfo, EM_SETSEL, lStrInfoNbrS, lStrInfoNbrE);
			udtPF.cbSize = sizeof(udtPF);
			udtPF.dwMask = PFM_NUMBERING | PFM_NUMBERINGSTART;
			udtPF.wNumbering = 2;
			udtPF.wNumberingStart = 1;
			SendMessage(hWnd_txtInfo, EM_SETPARAFORMAT, 0, (LPARAM)&udtPF);

			/* Set info box bold warning (no dialup modems!) */
			SendMessage(hWnd_txtInfo, EM_SETSEL, lStrInfoWarnS, lStrInfoWarnE);
			udtCF.cbSize = sizeof(udtCF);
			udtCF.dwMask = CFM_BOLD;
			udtCF.dwEffects = CFE_BOLD;
			SendMessage(hWnd_txtInfo, EM_SETCHARFORMAT, SCF_SELECTION, (LPARAM)&udtCF);

			SetForegroundWindow(hDlg);
			return true;

		/* Deal with user following hyperlinks */
		case WM_NOTIFY:
			switch (wParam)
			{
				case IDC_txtBanner:
					udtENLink = (ENLINK *)lParam;
					if ((udtENLink->nmhdr.code == EN_LINK) && (udtENLink->msg == WM_LBUTTONDOWN))
					{
						ShellExecute(hDlg, NULL, "http://muzzleflash.da.ru", NULL, NULL, SW_MAXIMIZE);
					}
					return true;
				case IDC_txtInfo:
					udtENLink = (ENLINK *)lParam;
					if ((udtENLink->nmhdr.code == EN_LINK) && (udtENLink->msg == WM_LBUTTONDOWN))
					{
						ShellExecute(hDlg, NULL, "http://netgroup-serv.polito.it/winpcap/", NULL, NULL, SW_MAXIMIZE);
					}
					return true;
			}
			break;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_cmdStart:
					EndDialog(hDlg, true);
					return true;
				case IDC_cmdExit:
					EndDialog(hDlg, false);
					return true;
			}
			break;
	}

	return false;
}
