#include <windows.h>
#include <malloc.h>
#include "Main.h"
#include "Functions.h"
#include "AODB.h"
#include "pcap.h"
#include "ethertypes.h"
#include "zlib.h"

/* Structures */
typedef struct adapter_struc
{
	char *strDevice;
	char *strName;
} udtAdapter_struc;
typedef struct ethernet_struc
{
	BYTE a_xDstAddr[6];
	BYTE a_xSrcAddr[6];
	union
	{
		unsigned short iLength;	// IEEE 802.3 frame
		unsigned short iType;	// Ethernet V2 frame
	};
} udtEthernet_struc;
typedef struct pppoe_struc
{
	BYTE xVersion:4,
	     xType:4;
	BYTE xCode;
	unsigned short iID;
	unsigned short iLength;
	unsigned short iProtocol;
} udtPPPoE_struc;
typedef struct ip_struc
{
	BYTE xVersion:4,
	     xHdrLen:4;
	BYTE xTOS;
	short iTotalLen;
	unsigned short iID;
	short iOffset;
	BYTE xTimeToLive;
	BYTE xProtocol;
	unsigned short iChecksum;
	unsigned long lSrcAddr;
	unsigned long lDstAddr;
} udtIP_struc;
typedef struct tcphdr_struc
{
	unsigned short iSrcPort;
	unsigned short iDstPort;
	unsigned long lSeqNbr;
	unsigned long lAckNbr;
	BYTE xOffset:4,
	     xUnused:4;
	BYTE xFlags;
	unsigned short iWindow;
	unsigned short iCheckSum;
	unsigned short iUrgPtr;
} udtTCPHdr_struc;
typedef struct playfield_struc
{
	short iNumber;
	char *strName;
} udtPlayfield_struc;
typedef struct findname_struc
{
	char *strStart;
	char *strEnd;
} udtFindName_struc;
typedef struct png_ihdr_struc
{
	unsigned long lWidth;
    unsigned long lHeight;
	BYTE xBitDepth;
	BYTE xColorType;
	BYTE xCompressMethod;
	BYTE xFilterMethod;
	BYTE xInterlaceMethod;
} udtPNGihdr_struc;
typedef struct png_sbit_struc
{
	BYTE xRed;
	BYTE xGreen;
	BYTE xBlue;
} udtPNGsbit_struc;
typedef struct png_pixel_struc
{
	BYTE xRed;
	BYTE xGreen;
	BYTE xBlue;
} udtPNGpixel_struc;

/* Callback function */
typedef void (CALLBACK *ShowMissionPtr)(long lMissionIdx, udtMission_struc *udtMission);
ShowMissionPtr ShowMission;

/* Function Declarations for internal use */
unsigned long WINAPI CaptureThread(void *pParam);
void OnPacket(u_char *, const struct pcap_pkthdr *, const u_char *);
char *MissionFind(unsigned long lMissionNbr);
long FindStr(BYTE *a_xBuf, unsigned long lBufLen, BYTE *a_xFind, unsigned long lFindLen);
char *MissionPF(unsigned long lMissionNbr);
char *MissionItem(unsigned long lMissionNbr);
BOOL GetAONameData(unsigned long lItemNo, char *strName, unsigned short *iQL, unsigned long *lIconNbr);
BITMAPINFO *GetAOIconData(unsigned long lIconNo);

/* Constants */
#define PPPoEPROTO_IP 0x0021
#define MSN_TIMEOUTSECS 5	// Allow 5 secs for all mission packets to arrive
#define MAX_ENET_PKT_LEN 0x5DC
static char strMissionStart[] = "credits were deducted from your account";
#define LEN_MDATAID 0x4
static BYTE a_xMissionDataID[LEN_MDATAID] = {0xAB, 0xAB, 0xAB, 0xAB};
#define CNT_PLAYFIELDS 46
static udtPlayfield_struc a_udtPlayfields[CNT_PLAYFIELDS] = {
	505, "Avalon",
	540, "Athen Old",
	545, "Athen West",
	550, "Athen Shire",
	551, "Wailing Wastes",
	560, "Mort",
	565, "Newland",
	566, "Newland City",
	567, "Newland Desert",
	570, "Perpetual Wastelands",
	585, "Aegean",
	586, "Wartorn Valley",
	590, "Central Artery Valley",
	595, "Deep Artery Valley",
	600, "Varmint Woods",
	605, "Belial Forest",
	610, "Southern Artery Valley",
	615, "Southern Fouls Hills",
	620, "Eastern Fouls Plain",
	625, "Milky Way",
	630, "Pleasant Meadows",
	635, "Stret East Bank",
	640, "Tir",
	646, "Tir County",
	647, "Greater Tir County",
	650, "Upper Stret East Bank",
	655, "Andromeda",
	665, "Broken Shores",
	670, "Clondyke",
	685, "Galway County",
	687, "Galway Shire",
	695, "Lush Fields",
	696, "Mutant Domain",
	700, "Omni-1 HQ",
	705, "Omni-1 Entertainment",
	710, "Omni-1 Trade",
	716, "Omni Forest",
	717, "Greater Omni Forest",
	730, "Rome Red",
	735, "Rome Blue",
	740, "Rome Green",
	760, "4 Holes",
	790, "Stret West Bank",
	791, "Holes in the Wall",
	795, "The Longest Road",
	800, "Borealis"
};
#define CNT_FINDNAME 25
static udtFindName_struc a_udtFindName[CNT_FINDNAME] = {
	"The enemy is in the process of creating a new prototype ", ". It is of utmost importance",
	"The enemy is currently making a new prototype ", ". It is of utmost importance",
	"We have reason to believe finding the ", " in ",
	"In this case it is the ", " that has gone missing.",
	"we have at last found a copy of the ", " in ",
	"According to our sources, the ", " found in ",
	"Last night, the ", " was stolen from a production facility",
	"Last night, one ", " was stolen from a production facility",
	"One of our ", " have been stolen from our ",
	"One ", " has been stolen from our ",
	"A hacker wiped the ", " from our database",
	"I am interested in obtaining a certain ", ". My contacts have",
	"I am interested in obtaining one ", ". My contacts have",
	"have developed a prototype ", ".  We would very,",
	"If we could steal the ", " from the enemy, we would",
	"you can find the entrance to the place where the ", " has been hidden.",
	"you can find the entrance to where the ", " has been hidden.",
	"Would you please find the ", " in ",
	"you might be able to find the ", ". Please bring it back to us",
	"Oh yeah, the ", " is set to blow up in",
	"who or where the traitor is, before you collect the ", " from ",
	"who or where he is, before you collect the ", " from ",
	"you might be able to find one ", ". Bring it back to us",
	"we intercepted a message that a prototype ", " will be moved from",
	"It is the ", ", please retrieve it "
};
#define LEN_PNGSIG 0x8
BYTE a_xPNGSig[LEN_PNGSIG] = {0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A};
#define AODB_MAX_NAME_LEN 128
static char strNoAdapterName[] = "Unnamed Adapter";
static BITMAPFILEHEADER bmiBMPhdr = {0x4D42, 0x1B36, 0, 0, 0x36};
static char strID_AOPath[] = "[AOPath]";

/* Variables */
long lAdapterCnt;
long lAdapterIdx;
HANDLE hThread;
DWORD dwThreadID;
BOOL bIsWinNT;
BOOL bMissionStream;
char strError[PCAP_ERRBUF_SIZE];
char strPFNbr[20];
char strItemName1[AODB_MAX_NAME_LEN];
char strItemName2[AODB_MAX_NAME_LEN];
char *strItemName;
char strDebugFile[MAX_PATH];
udtAdapter_struc a_udtAdapter[10];
udtMission_struc a_udtMission[AO_MISSION_MAX];
pcap_t *fpPcap;

/* Function to be called on startup */
void StartUp(void)
{
	DWORD dwVersion;
	DWORD dwWindowsMajorVersion;
	
	/* Initialise variables */
	bMissionStream = false;
	ZeroMemory(a_udtMission, AO_MISSION_MAX * sizeof(udtMission_struc));

	/* Determine version of Windows */
	dwVersion = GetVersion();
	dwWindowsMajorVersion =  (DWORD)(LOBYTE(LOWORD(dwVersion)));
	if (dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4)
		bIsWinNT = false;
	else
		bIsWinNT = true;
}

/* Find devices that we can use */
long FindDevices(void)
{
	long lLoop;
	char *strDevices;
	char *strDevASCII;
	WCHAR *strDevUNICODE;

	/* Initialise */
	lAdapterCnt = 0;
	
	/* Get start of the devices string */
	strDevices = pcap_lookupdev(strError);

	/* Windows NT/2000/XP - Unicode device */
	if (bIsWinNT)
	{
		lLoop = 0;
		strDevUNICODE = (WCHAR *)strDevices;
		if (strDevUNICODE[lLoop] != 0 || strDevUNICODE[lLoop+1] != 0)
		{
			a_udtAdapter[lAdapterCnt].strDevice = (char *)strDevUNICODE;
			lAdapterCnt += 1;
			while (strDevUNICODE[lLoop] != 0 || strDevUNICODE[lLoop+1] != 0)
			{
				if (strDevUNICODE[lLoop] == 0)
				{
					a_udtAdapter[lAdapterCnt].strDevice = (char *)(&(strDevUNICODE[lLoop+1]));
					lAdapterCnt += 1;
				}
				lLoop++;
			}
			strDevUNICODE += (lLoop + 2);
		}
		strDevices = (char *)strDevUNICODE;
	}
	
	/* Windows 95/98/ME - ASCII device */
	else
	{
		lLoop = 0;
		strDevASCII = strDevices;
		if (strDevASCII[lLoop] != 0 || strDevASCII[lLoop+1] != 0)
		{
			a_udtAdapter[lAdapterCnt].strDevice = strDevASCII;
			lAdapterCnt += 1;
			while (strDevASCII[lLoop] != 0 || strDevASCII[lLoop+1] != 0)
			{
				if (strDevASCII[lLoop] == 0)
				{
					a_udtAdapter[lAdapterCnt].strDevice = &(strDevASCII[lLoop+1]);
					lAdapterCnt += 1;
				}
				lLoop++;
			}
			strDevASCII += (lLoop + 2);
		}
		strDevices = strDevASCII;
	}
	
	/* Now handle the adapter names - these are ASCII */
	if (lAdapterCnt > 0)
	{
		for (lLoop = 0; lLoop < lAdapterCnt; lLoop++)
		{
			a_udtAdapter[lLoop].strName = strDevices;
			if (a_udtAdapter[lLoop].strName[0] == 0)
				a_udtAdapter[lLoop].strName = strNoAdapterName;
			while (strDevices[0] != 0)
			{
				strDevices++;
			}
			strDevices++;
		}
	}

	/* Return the number of adapters found */
	return lAdapterCnt;
}

/* Find devices that we can use */
char *GetDeviceName(long lIndex)
{
	return a_udtAdapter[lIndex].strName;
}

/* Set the active device to be used */
void SetActiveDevice(long lIndex)
{
	lAdapterIdx = lIndex;
}

/* Begin capture */
BOOL BeginCapture(void *ShowMissionCallback)
{
	/* Open adapter */
	if ((fpPcap= pcap_open_live(a_udtAdapter[lAdapterIdx].strDevice, 0x10000, true, 20, strError) ) == NULL)
		return false;

	/* Assign callback function to display mission */
	ShowMission = (ShowMissionPtr)ShowMissionCallback;

	/* Create the capture thread */
	hThread = CreateThread(NULL, 0, &CaptureThread, NULL, 0, &dwThreadID);
	if (!hThread)
		return false;

	return true;
}

/* The capture thread */
unsigned long WINAPI CaptureThread(void *pParam)
{
	/* Start the capture loop */
	pcap_loop (fpPcap, 0, OnPacket, NULL);

	return 1;
}

/* Callback function called by libpcap for every incoming packet */
void OnPacket(u_char *fpDump, const struct pcap_pkthdr *objHeader, const u_char *objData)
{
	static unsigned short iSrcPort;
	static unsigned short iDstPort;
	long lPos;
	long lPktRem;
	long lLoop;
	long lMissionCnt;
	long lDataRem;
	unsigned long lPktLen;
	unsigned long lMisDesc;
	unsigned long lMisDescLen;
	static unsigned long lSrcAddr;
	static unsigned long lDstAddr;
	static unsigned long lMaxPktLen;
	static unsigned long lMissionLen;
	static unsigned long lMissionPktLen;
	__int64 qwCurrentTime;
	static __int64 qwTimeout;
	char *strItemName;
	char *strLocName;
	BYTE *a_xPkt;
	BYTE *a_xMisData;
	BYTE *a_xMisDesc;
	BYTE *a_xMisDescLen;
	BOOL bDescFound;
	static BYTE a_xMissionPkt[0x10000];
	SYSTEMTIME stCurrentTime;
	FILETIME ftCurrentTime;
	udtEthernet_struc *udtENet;
	udtPPPoE_struc *udtPPPoE;
	udtIP_struc *udtIP;
	udtTCPHdr_struc *udtTCPHdr;
	FILE *fpDebug;

	/* Write captured packet to debug file */
	if (lDebug & DBG_PKT)
	{
		sprintf(strDebugFile, "%sDebug_AllPackets.DAT", strAOMDPath);
		fpDebug = fopen(strDebugFile, "a+b");
		fwrite(objData, sizeof(BYTE), objHeader->len, fpDebug);
		fwrite("****************", sizeof(char), 0x10 - (objHeader->len % 0x10), fpDebug);
		fwrite("****************", sizeof(char), 0x10, fpDebug);
		fclose(fpDebug);
	}

	/* Initialise */
	lPktLen = objHeader->len;
	a_xPkt = (BYTE *)objData;
	GetSystemTime(&stCurrentTime);
	SystemTimeToFileTime(&stCurrentTime, &ftCurrentTime);
	memcpy(&qwCurrentTime, &ftCurrentTime, sizeof(__int64));

	/* Check ethertype and find IP packet header */
	udtENet = (udtEthernet_struc *)a_xPkt;
	if ((a_xPkt + lPktLen) < ((BYTE *)udtENet + sizeof(udtEthernet_struc)))
		return;		// Ensure packet is large enough
	switch (SwapEndianS(udtENet->iType))
	{
		case ETHERTYPE_IP:
			udtIP = (udtIP_struc *)((BYTE *)udtENet + sizeof(udtEthernet_struc));
			break;
		
		case ETHERTYPE_PPPOE:
			udtPPPoE = (udtPPPoE_struc *)((BYTE *)udtENet + sizeof(udtEthernet_struc));
			if ((a_xPkt + lPktLen) < ((BYTE *)udtPPPoE + sizeof(udtPPPoE_struc)))
				return;		// Ensure packet is large enough
			if (SwapEndianS(udtPPPoE->iProtocol) != PPPoEPROTO_IP)
				return;		// Ensure it is IP protocol

			udtIP = (udtIP_struc *)((BYTE *)udtPPPoE + sizeof(udtPPPoE_struc));
			break;
		
		default:	// Unsupported ethertype (Ethernet II), or it is length (IEEE 802.3)... try IP style
			udtIP = (udtIP_struc *)((BYTE *)udtENet + sizeof(udtEthernet_struc));
			break;
	}

	/* Check for TCP protocol */
	if ((a_xPkt + lPktLen) < ((BYTE *)udtIP + sizeof(udtIP_struc)))
		return;		// Ensure packet is large enough
	if (udtIP->xProtocol != IPPROTO_TCP)
		return;		// Exit if not TCP

	/* Address the TCP header */
	udtTCPHdr = (udtTCPHdr_struc *)((BYTE *)udtIP + sizeof(udtIP_struc));
	if ((a_xPkt + lPktLen) < ((BYTE *)udtTCPHdr + sizeof(udtTCPHdr_struc)))
		return;		// Ensure packet is large enough

	/* Determine where to start looking */
	if (bMissionStream)
	{
		/* Ensure source/dest IP address and port matches the first packet */
		if ((lSrcAddr != udtIP->lSrcAddr) || (lDstAddr != udtIP->lDstAddr) || (iSrcPort != udtTCPHdr->iSrcPort) || (iDstPort != udtTCPHdr->iDstPort))
		{
			if (qwCurrentTime > qwTimeout)
				goto OnPacket_Timeout;	// Previous mission stream timed out
			
			return;
		}
		
		/* Set position to start of data continuation */
		lPos = ((BYTE *)udtTCPHdr + sizeof(udtTCPHdr_struc)) - a_xPkt;
	}
	else
	{
		/* Find credit deduction, marks start of missions */
		lPos = FindStr(a_xPkt, lPktLen, (BYTE *)strMissionStart, strlen(strMissionStart));
		if (lPos < 0)
			return;
		lPos += strlen(strMissionStart);

		/* Flag start of mission stream */
		bMissionStream = true;

		/* Save source/dest IP address and port, and max packet size */
		lSrcAddr = udtIP->lSrcAddr;
		lDstAddr = udtIP->lDstAddr;
		iSrcPort = udtTCPHdr->iSrcPort;
		iDstPort = udtTCPHdr->iDstPort;
		lMaxPktLen = lPktLen;

		/* Set timeout value */
		qwTimeout = qwCurrentTime + ((__int64)MSN_TIMEOUTSECS * (__int64)10000000);

		/* Initialise mission buffer */
		ZeroMemory(a_xMissionPkt, sizeof(a_xMissionPkt));
		lMissionLen = 0;
	}

	/* Save packet data to mission buffer */
	lPktRem = lPktLen - lPos;
	if ((lMissionLen + lPktRem) > sizeof(a_xMissionPkt))
		lPktRem = lMissionPktLen - lMissionLen;		// Avoid overflow
	memcpy (&(a_xMissionPkt[lMissionLen]), &(a_xPkt[lPos]), lPktRem);
	lMissionLen += lPktRem;

	/* If packet is maximum size, assume continuation in next packet */
	if (lPktLen >= lMaxPktLen)
		return;
	
OnPacket_Timeout:
	/* Turn off mission stream indicator */
	bMissionStream = false;

	/* Write mission data to debug file */
	if (lDebug & DBG_MSN)
	{
		sprintf(strDebugFile, "%sDebug_MissionData.DAT", strAOMDPath);
		fpDebug = fopen(strDebugFile, "a+b");
		fwrite (a_xMissionPkt, sizeof(BYTE), lMissionLen, fpDebug);
		fwrite("****************", sizeof(char), 0x10 - (lMissionLen % 0x10), fpDebug);
		fwrite("****************", sizeof(char), 0x10, fpDebug);
		fclose (fpDebug);
	}

	/* Build mission data array */
	lMissionCnt = 0;
	a_xMisData = a_xMissionPkt;
	lDataRem = lMissionLen;
	for (lLoop = 0; lLoop < AO_MISSION_MAX; lLoop += 1)
	{
		/* Find mission data marker */
		lPos = FindStr(a_xMisData, lDataRem, a_xMissionDataID, LEN_MDATAID);
		if (lPos < 0)
			break;
		a_xMisData += lPos;
		lDataRem -= lPos;

		/* Look ahead, the 0xAB may be part of cash value */
		while (memcmp((a_xMisData + 1), a_xMissionDataID, LEN_MDATAID) == 0)
		{
			a_xMisData++;
			lDataRem--;
		}
		
		/* Shift back to cash */
		a_xMisData -= 0x4;
		lDataRem += 0x4;

		/* Check enough data remaining */
		if (lDataRem < 0xC8)	// 0xC7 is last value we need (location)
			break;

		/* Store cash reward */
		a_udtMission[lMissionCnt].lCash = SwapEndianL(*(unsigned long *)a_xMisData);

		/* Store item reward */
		free(a_udtMission[lMissionCnt].strItemName);
		a_udtMission[lMissionCnt].lItem1 = SwapEndianL(*(unsigned long *)(a_xMisData + 0x18));
		a_udtMission[lMissionCnt].lItem2 = SwapEndianL(*(unsigned long *)(a_xMisData + 0x1C));
		a_udtMission[lMissionCnt].iQL = (short)*(a_xMisData + 0x23);
		strItemName = MissionItem(lMissionCnt);
		a_udtMission[lMissionCnt].iItemNameLen = strlen(strItemName) + 1;
		a_udtMission[lMissionCnt].strItemName = (char *)malloc(a_udtMission[lMissionCnt].iItemNameLen);
		memcpy(a_udtMission[lMissionCnt].strItemName, strItemName, a_udtMission[lMissionCnt].iItemNameLen);

		/* Fetch item icon */
		if (a_udtMission[lMissionCnt].lIconNbr)
			a_udtMission[lMissionCnt].bmiIcon = GetAOIconData(a_udtMission[lMissionCnt].lIconNbr);
		else
			a_udtMission[lMissionCnt].bmiIcon = NULL;
		
		/* Fetch mission type icon */
		a_udtMission[lMissionCnt].lMsnTypeIconNbr = SwapEndianL(*(unsigned long *)(a_xMisData + 0x44));

		/* Store location */
		free(a_udtMission[lMissionCnt].strPFName);
		a_udtMission[lMissionCnt].iPF = SwapEndianS(*(unsigned short *)(a_xMisData + 0xC6));
		strLocName = MissionPF(lMissionCnt);
		a_udtMission[lMissionCnt].iPFNameLen = strlen(strLocName) + 1;
		a_udtMission[lMissionCnt].strPFName = (char *)malloc(a_udtMission[lMissionCnt].iPFNameLen);
		memcpy(a_udtMission[lMissionCnt].strPFName, strLocName, a_udtMission[lMissionCnt].iPFNameLen);

		/* Search backwards for mission description */
		a_xMisDesc = a_xMisData - 0xC;
		a_xMisDescLen = a_xMisDesc - 0x4;	// Length (long) is prior to string
		lMisDesc = 0;
		bDescFound = false;
		while (a_xMisDescLen > a_xMissionPkt)	// Ensure we don't search beyond start of data
		{
			/* Fetch value that we need to test if it is length */
			lMisDescLen = SwapEndianL(*(unsigned long *)a_xMisDescLen);

			/* If value matches the calculated length, we found the description */
			if (lMisDescLen == lMisDesc)
			{
				bDescFound = true;
				break;
			}

			/* Adjust pointers and length for next test */
			a_xMisDescLen--;
			a_xMisDesc--;
			lMisDesc++;
		}

		/* Clear mission description and find item, then process if found */
		free(a_udtMission[lMissionCnt].strDesc);
		a_udtMission[lMissionCnt].strDesc = NULL;
		a_udtMission[lMissionCnt].lDescLen = 0;
		free(a_udtMission[lMissionCnt].strFindName);
		a_udtMission[lMissionCnt].strFindName = NULL;
		a_udtMission[lMissionCnt].iFindNameLen = 0;
		if (bDescFound)
		{
			/* Store mission description */
			a_udtMission[lMissionCnt].lDescLen = lMisDesc;
			a_udtMission[lMissionCnt].strDesc = (char *)malloc(lMisDesc);
			memcpy(a_udtMission[lMissionCnt].strDesc, a_xMisDesc, lMisDesc);
			
			/* Determine 'find item' name (if any) */
			if ((a_udtMission[lMissionCnt].lMsnTypeIconNbr == MSN_FINDITEM) || (a_udtMission[lMissionCnt].lMsnTypeIconNbr == MSN_RETURNITEM))
			{
				a_udtMission[lMissionCnt].strFindName = MissionFind(lMissionCnt);
				if (a_udtMission[lMissionCnt].strFindName)
					a_udtMission[lMissionCnt].iFindNameLen = strlen(a_udtMission[lMissionCnt].strFindName);
			}
		}

		/* Increment mission count */
		lMissionCnt += 1;

		/* Bump past data for next search */
		a_xMisData += 0xF1;
		lDataRem -= 0xF1;
		if (lDataRem < 1)
			break;	// Ensure we don't overflow
	}

	/* If no missions, exit */
	if (!lMissionCnt)
		return;

	/* Update display for each mission using callback */
	for (lLoop = 0; lLoop < lMissionCnt; lLoop += 1)
	{
		ShowMission(lLoop, &(a_udtMission[lLoop]));
	}

	/* Null display for unused missions */
	for (lLoop = lMissionCnt; lLoop < AO_MISSION_MAX; lLoop += 1)
	{
		ShowMission(lLoop, NULL);
	}
}

/* Return mission item to be found */
char *MissionFind(unsigned long lMissionNbr)
{
	long lLoop;
	long lPosStart;
	long lRem;
	long lLength;
	BOOL bFoundItem;
	char *strStart;
	char *strFindName;

	/* Initialise */
	bFoundItem = false;

	/* Search each of the mission 'find item' text matches */
	for (lLoop = 0; lLoop < CNT_FINDNAME; lLoop++)
	{
		lPosStart = FindStr((BYTE *)a_udtMission[lMissionNbr].strDesc, a_udtMission[lMissionNbr].lDescLen, (BYTE *)a_udtFindName[lLoop].strStart, strlen(a_udtFindName[lLoop].strStart));
		if (lPosStart >= 0)
		{
			/* Set ptr to start of find item name */
			strStart = a_udtMission[lMissionNbr].strDesc + lPosStart + strlen(a_udtFindName[lLoop].strStart);

			/* Search the remainder for trailing text */
			lRem = a_udtMission[lMissionNbr].lDescLen - (lPosStart + strlen(a_udtFindName[lLoop].strStart));
			lLength = FindStr((BYTE *)strStart, lRem, (BYTE *)a_udtFindName[lLoop].strEnd, strlen(a_udtFindName[lLoop].strEnd));
			if (lLength >= 0)
			{
				bFoundItem = true;
				break;
			}
		}
	}

	/* Return copy of item name, or null if not found */
	if (bFoundItem)
	{
		strFindName = (char *)malloc(lLength + 1);
		memcpy(strFindName, strStart, lLength);
		strFindName[lLength] = 0;
		return strFindName;
	}
	else
		return NULL;
}

/* Scan for a particular sequence of bytes */
long FindStr(BYTE *a_xBuf, unsigned long lBufLen, BYTE *a_xFind, unsigned long lFindLen)
{
	long lLoop;
	long lMax;
	BYTE *a_xBufPtr;
	
	a_xBufPtr = a_xBuf;
	lMax = (long)lBufLen - (long)lFindLen;
	for (lLoop = 0; lLoop <= lMax; lLoop += 1)
	{
		if (memcmp(a_xBufPtr,a_xFind,lFindLen) == 0)
			break;
		a_xBufPtr += 1;
	}
	if (lLoop > lMax)
		return -1;
	else
		return lLoop;
}

/* Return mission PlayField */
char *MissionPF(unsigned long lMissionNbr)
{
	long lLoop;

	/* Find descriptive name for playfield number */
	for (lLoop = 0; lLoop < CNT_PLAYFIELDS; lLoop++)
	{
		if (a_udtPlayfields[lLoop].iNumber == a_udtMission[lMissionNbr].iPF)
			return a_udtPlayfields[lLoop].strName;
	}

	/* Description not found, return number only */
	sprintf(strPFNbr, "PF-%d", a_udtMission[lMissionNbr].iPF);
	return strPFNbr;
}

/* Fetch the item name from AO Database */
char *MissionItem(unsigned long lMissionNbr)
{
	unsigned short iQL1, iQL2;
	unsigned long lIcon1, lIcon2;

	if (!a_udtMission[lMissionNbr].lItem1)
		goto FetchItemName_Err_NotFound;
	
	/* Get description for item number 1 */
	if (!GetAONameData(a_udtMission[lMissionNbr].lItem1, strItemName1, &iQL1, &lIcon1))
		goto FetchItemName_Err_NotFound;

	/* If no item number 2, then just keep the first description */
	if (!a_udtMission[lMissionNbr].lItem2)
	{
		strItemName = strItemName1;
		a_udtMission[lMissionNbr].lIconNbr = lIcon1;
	}

	/* Item number 2 exists, must interpolate */
	else
	{
		if (!GetAONameData(a_udtMission[lMissionNbr].lItem2, strItemName2, &iQL2, &lIcon2))
			goto FetchItemName_Err_NotFound;
		if (abs(a_udtMission[lMissionNbr].iQL - iQL1) < abs(iQL2 - a_udtMission[lMissionNbr].iQL))
		{
			strItemName = strItemName1;
			a_udtMission[lMissionNbr].lIconNbr = lIcon1;
		}
		else
		{
			strItemName = strItemName2;
			a_udtMission[lMissionNbr].lIconNbr = lIcon2;
		}
	}

	/* Success, return item name */
	return strItemName;

FetchItemName_Err_NotFound:
	sprintf(strItemName, "Unknown (%X:%X)", a_udtMission[lMissionNbr].lItem1, a_udtMission[lMissionNbr].lItem2);
	a_udtMission[lMissionNbr].lIconNbr = 0;
	return strItemName;
}

/* Get item name from AO Database */
BOOL GetAONameData(unsigned long lItemNo, char *strName, unsigned short *iQL, unsigned long *lIconNbr)
{
	unsigned long lDataLen;
	unsigned long lStat;
	unsigned long lValue;
	unsigned long lNameLen;
	unsigned long lDescLen;
	BYTE *a_xData;
	BYTE *a_xDataPtr;
	FILE *fpDebug;

	/* Make name and desc blank strings, set QL to zero */
	*iQL = 0;
	*lIconNbr = 0;
	strName[0] = 0;

	/* Read data for this item */
	lDataLen = ReadDataChunk(AODB_TYP_ITEM, lItemNo, &a_xData);
	if (!lDataLen)
		return false;
	
	/* Items are encoded - so we must decode the data */
	DecodeDataChunk(a_xData, lDataLen);

	/* Write item data to file */
	if (lDebug & DBG_ITM)
	{
		sprintf(strDebugFile, "%sDebug_ItemData.DAT", strAOMDPath);
		fpDebug = fopen(strDebugFile, "a+b");
		fwrite(a_xData, sizeof(BYTE), lDataLen, fpDebug);
		fwrite("****************", sizeof(char), 0x10 - (lDataLen % 0x10), fpDebug);
		fwrite("****************", sizeof(char), 0x10, fpDebug);
		fclose(fpDebug);
	}

	/* Set ptr to first slot */
	a_xDataPtr = a_xData + 0x24;

	/* Process the slots we require */
	while (a_xDataPtr + 8 <= a_xData + lDataLen)
	{
		lStat = *((unsigned long *)a_xDataPtr);
		lValue = *((unsigned long *)a_xDataPtr+1);
		a_xDataPtr += 0x8;
		
		switch (lStat)
		{
			/* Quality */
			case 0x36:
				*iQL = (unsigned short)lValue;
				break;
			
			/* Icon */
			case 0x4F:
				*lIconNbr = lValue;
				break;
			
			/* Item name and description */
			case 0x15:
				if (lValue == 0x21)
				{
					lNameLen = *((unsigned short *)a_xDataPtr);
					lDescLen = *((unsigned short *)a_xDataPtr+1);
					a_xDataPtr += 0x4;

					/* Store the item name */
					if (lNameLen > AODB_MAX_NAME_LEN)
						lNameLen = AODB_MAX_NAME_LEN;
					if (a_xDataPtr + lNameLen > a_xData + lDataLen)
						goto GetAONameData_Exit_Fail;	// Abort - not enough data left
					memcpy(strName, a_xDataPtr, lNameLen);
					strName[lNameLen] = 0;
					
					/* Successfully retrieved name, return */
					ReleaseDataChunk(&a_xData);
					return true;
				}
				break;
		}
	}

GetAONameData_Exit_Fail:	// This exit frees retrieved data
	ReleaseDataChunk(&a_xData);
	return false;
}

/* Get item icon from AO Database */
BITMAPINFO *GetAOIconData(unsigned long lIconNo)
{
	unsigned long lDataLen;
	unsigned long lChunkLen;
	unsigned long lPNGLen;
	unsigned long lLoop;
	unsigned long lLoop2;
	unsigned long lBytesPerRow;
	unsigned long lDIBsize;
	unsigned long lPNGDataLen;
	unsigned long lPNGImageLen;
	unsigned long lPNGRowOffset;
	BYTE xFilter;
	BYTE *a_xData;
	BYTE *a_xPNG;
	BYTE *a_xPNGChunk;
	BYTE *a_xPNGData;
	BYTE *a_xPNGImage;
	BYTE *a_xPNGRow, *a_xPNGRowPrev;
	udtPNGpixel_struc *udtCLinCByt, *udtCLinPByt, *udtPLinCByt, *udtPLinPByt;
	udtPNGpixel_struc *udtPNGpixel;
	RGBTRIPLE *rgbDIBpixel;
	char strChunkID[5];
	udtPNGihdr_struc *udtPNGihdr;
	udtPNGsbit_struc *udtPNGsbit;
	BITMAPINFO *bmiDIB;
	FILE *fpDebug;

	/* Initialise */
	a_xData = NULL;
	bmiDIB = NULL;
	a_xPNGImage = NULL;
	a_xPNGRowPrev = NULL;

	/* Read data for this item */
	lDataLen = ReadDataChunk(AODB_TYP_ICON, lIconNo, &a_xData);
	if (!lDataLen)
		goto GetAOIconData_Exit_Fail;

	/* Check it contains a valid PNG */
	a_xPNG = a_xData + 0x18;
	lPNGLen = lDataLen - 0x18;
	if (memcmp(a_xPNG, a_xPNGSig, LEN_PNGSIG) != 0)
		goto GetAOIconData_Exit_Fail;
	a_xPNGChunk = a_xPNG + 0x8;		// Point to first chunk

	/* Write PNG icon to file */
	if (lDebug & DBG_ICN)
	{
		sprintf(strDebugFile, "%sDebug_IconsPNG.DAT", strAOMDPath);
		fpDebug = fopen(strDebugFile, "a+b");
		fwrite(a_xPNG, sizeof(BYTE), lPNGLen, fpDebug);
		fwrite("****************", sizeof(char), 0x10 - (lPNGLen % 0x10), fpDebug);
		fwrite("****************", sizeof(char), 0x10, fpDebug);
		fclose(fpDebug);
	}
	
	/* Check IHDR chunk - Start of PNG, contains icon properties */
	lChunkLen = SwapEndianL(*(unsigned long *)(a_xPNGChunk));
	lChunkLen += 0xC;
	memcpy(strChunkID, (a_xPNGChunk + 0x4), 4);
	strChunkID[4] = 0;
	if (stricmp(strChunkID, "IHDR") != 0)
		goto GetAOIconData_Exit_Fail;
	udtPNGihdr = (udtPNGihdr_struc *)(a_xPNGChunk + 0x8);
	udtPNGihdr->lWidth = SwapEndianL(udtPNGihdr->lWidth);	// Fix endian
	udtPNGihdr->lHeight = SwapEndianL(udtPNGihdr->lHeight);	// Fix endian
	a_xPNGChunk += lChunkLen;	// Bump to next chunk

	/* Ensure PNG properties are what we expect from AO */
	if ((udtPNGihdr->lWidth != 48) || (udtPNGihdr->lHeight != 48))
		goto GetAOIconData_Exit_Fail;	// not a 48x48 image
	if ((udtPNGihdr->xBitDepth != 8) || (udtPNGihdr->xColorType != 2))
		goto GetAOIconData_Exit_Fail;	// not 24bit RGB
	if ((udtPNGihdr->xCompressMethod != 0) || (udtPNGihdr->xFilterMethod != 0))
		goto GetAOIconData_Exit_Fail;	// non-standard compression or filter
	if (udtPNGihdr->xInterlaceMethod != 0)
		goto GetAOIconData_Exit_Fail;	// must not be interlaced

	/* Check SBIT chunk - Significant bits */
	lChunkLen = SwapEndianL(*(unsigned long *)(a_xPNGChunk));
	lChunkLen += 0xC;
	memcpy(strChunkID, (a_xPNGChunk + 0x4), 4);
	strChunkID[4] = 0;
	if (stricmp(strChunkID, "SBIT") != 0)
		goto GetAOIconData_Exit_Fail;
	udtPNGsbit = (udtPNGsbit_struc *)(a_xPNGChunk + 0x8);
	a_xPNGChunk += lChunkLen;	// Bump to next chunk

	/* Check IDAT chunk - Contains the icon data*/
	lPNGDataLen = SwapEndianL(*(unsigned long *)(a_xPNGChunk));
	lChunkLen = lPNGDataLen + 0xC;
	memcpy(strChunkID, (a_xPNGChunk + 0x4), 4);
	strChunkID[4] = 0;
	if (stricmp(strChunkID, "IDAT") != 0)
		goto GetAOIconData_Exit_Fail;
	a_xPNGData = a_xPNGChunk + 0x8;
	a_xPNGChunk += lChunkLen;	// Bump to next chunk

	/* Allocate bitmap */
	lBytesPerRow = (((udtPNGihdr->lWidth * 24) + 31) / 32) * 4;
	lDIBsize = sizeof(BITMAPINFOHEADER) + (udtPNGihdr->lHeight * lBytesPerRow);
	bmiDIB = (BITMAPINFO *)malloc(lDIBsize);
	if (!bmiDIB)
		goto GetAOIconData_Exit_Fail;

	/* Fill in bitmap header details */
	bmiDIB->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmiDIB->bmiHeader.biWidth = udtPNGihdr->lWidth;
	bmiDIB->bmiHeader.biHeight = udtPNGihdr->lHeight;
	bmiDIB->bmiHeader.biPlanes = 1;
	bmiDIB->bmiHeader.biBitCount = 24;
	bmiDIB->bmiHeader.biCompression = BI_RGB;
	bmiDIB->bmiHeader.biSizeImage = 0;
	bmiDIB->bmiHeader.biXPelsPerMeter = 0;
	bmiDIB->bmiHeader.biYPelsPerMeter = 0;
	bmiDIB->bmiHeader.biClrUsed = 0;
	bmiDIB->bmiHeader.biClrImportant = 0;

	/* Decompress the PNG image data using ZLib */
	lPNGImageLen = udtPNGihdr->lHeight * (lBytesPerRow + 1);
	a_xPNGImage = (BYTE *)malloc(lPNGImageLen);
	if (uncompress(a_xPNGImage, &lPNGImageLen, a_xPNGData, lPNGDataLen) != Z_OK)
		goto GetAOIconData_Exit_Fail;

	/* Allocate previous row buffer and init to zero */
	a_xPNGRowPrev = (BYTE *)malloc(lBytesPerRow);
	ZeroMemory(a_xPNGRowPrev, lBytesPerRow);

	/* Filter each row and copy to bitmap */
	for (lLoop = 0; lLoop < udtPNGihdr->lHeight; lLoop++)
	{
		lPNGRowOffset = lLoop * (lBytesPerRow + 1);
		xFilter = a_xPNGImage[lPNGRowOffset];
		a_xPNGRow = a_xPNGImage + lPNGRowOffset + 1;
		switch (xFilter)
		{
			/* Filter 0 - None */
			case 0:
				break;
			
			/* Filter 1 - Sub */
			case 1:
				udtCLinCByt = (udtPNGpixel_struc *)a_xPNGRow + 1;
				udtCLinPByt = (udtPNGpixel_struc *)a_xPNGRow;
				for (lLoop2 = 1; lLoop2 < (lBytesPerRow/3); lLoop2++)
				{
					udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + (int)(udtCLinPByt->xRed)) & 0xFF);
					udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + (int)(udtCLinPByt->xGreen)) & 0xFF);
					udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + (int)(udtCLinPByt->xBlue)) & 0xFF);
					udtCLinCByt++;
					udtCLinPByt++;
				}
				break;
			
			/* Filter 2 - Up */
			case 2:
				udtCLinCByt = (udtPNGpixel_struc *)a_xPNGRow;
				udtPLinCByt = (udtPNGpixel_struc *)a_xPNGRowPrev;
				for (lLoop2 = 0; lLoop2 < (lBytesPerRow/3); lLoop2++)
				{
					udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + (int)(udtPLinCByt->xRed)) & 0xFF);
					udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + (int)(udtPLinCByt->xGreen)) & 0xFF);
					udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + (int)(udtPLinCByt->xBlue)) & 0xFF);
					udtCLinCByt++;
					udtPLinCByt++;
				}
				break;
			
			/* Filter 3 - Average */
			case 3:
				udtCLinCByt = (udtPNGpixel_struc *)a_xPNGRow;
				udtPLinCByt = (udtPNGpixel_struc *)a_xPNGRowPrev;
				udtCLinPByt = (udtPNGpixel_struc *)a_xPNGRow;
				udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + ((int)(udtPLinCByt->xRed) >> 1)) & 0xFF);
				udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + ((int)(udtPLinCByt->xGreen) >> 1)) & 0xFF);
				udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + ((int)(udtPLinCByt->xBlue) >> 1)) & 0xFF);
				udtCLinCByt++;
				udtPLinCByt++;
				for (lLoop2 = 0; lLoop2 < (lBytesPerRow/3) - 1; lLoop2++)
				{
					udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + ((int)(udtPLinCByt->xRed + udtCLinPByt->xRed) >> 1)) & 0xFF);
					udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + ((int)(udtPLinCByt->xGreen + udtCLinPByt->xGreen) >> 1)) & 0xFF);
					udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + ((int)(udtPLinCByt->xBlue + udtCLinPByt->xBlue) >> 1)) & 0xFF);
					udtCLinCByt++;
					udtPLinCByt++;
					udtCLinPByt++;
				}
				break;
			
			/* Filter 4 - Paeth */
			case 4:
				udtCLinCByt = (udtPNGpixel_struc *)a_xPNGRow;
				udtPLinCByt = (udtPNGpixel_struc *)a_xPNGRowPrev;
				udtCLinPByt = (udtPNGpixel_struc *)a_xPNGRow;
				udtPLinPByt = (udtPNGpixel_struc *)a_xPNGRowPrev;
				udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + (int)(udtPLinCByt->xRed)) & 0xFF);
				udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + (int)(udtPLinCByt->xGreen)) & 0xFF);
				udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + (int)(udtPLinCByt->xBlue)) & 0xFF);
				udtCLinCByt++;
				udtPLinCByt++;
				for (lLoop2 = 0; lLoop2 < (lBytesPerRow/3) - 1; lLoop2++)
				{
					int lCLinPByt_R, lPLinCByt_R, lPLinPByt_R, lPaethA_R, lPaethB_R, lPaethC_R, lPaeth_R;
					lCLinPByt_R = udtCLinPByt->xRed;
					lPLinCByt_R = udtPLinCByt->xRed;
					lPLinPByt_R = udtPLinPByt->xRed;
					lPaeth_R = lPLinCByt_R - lPLinPByt_R;
					lPaethC_R = lCLinPByt_R - lPLinPByt_R;
					lPaethA_R = lPaeth_R < 0 ? -lPaeth_R : lPaeth_R;
					lPaethB_R = lPaethC_R < 0 ? -lPaethC_R : lPaethC_R;
					lPaethC_R = (lPaeth_R + lPaethC_R) < 0 ? -(lPaeth_R + lPaethC_R) : lPaeth_R + lPaethC_R;
					lPaeth_R = (lPaethA_R <= lPaethB_R && lPaethA_R <= lPaethC_R) ? lCLinPByt_R : (lPaethB_R <= lPaethC_R) ? lPLinCByt_R : lPLinPByt_R;
					udtCLinCByt->xRed = (BYTE)(((int)(udtCLinCByt->xRed) + lPaeth_R) & 0xFF);
					
					int lCLinPByt_G, lPLinCByt_G, lPLinPByt_G, lPaethA_G, lPaethB_G, lPaethC_G, lPaeth_G;
					lCLinPByt_G = udtCLinPByt->xGreen;
					lPLinCByt_G = udtPLinCByt->xGreen;
					lPLinPByt_G = udtPLinPByt->xGreen;
					lPaeth_G = lPLinCByt_G - lPLinPByt_G;
					lPaethC_G = lCLinPByt_G - lPLinPByt_G;
					lPaethA_G = lPaeth_G < 0 ? -lPaeth_G : lPaeth_G;
					lPaethB_G = lPaethC_G < 0 ? -lPaethC_G : lPaethC_G;
					lPaethC_G = (lPaeth_G + lPaethC_G) < 0 ? -(lPaeth_G + lPaethC_G) : lPaeth_G + lPaethC_G;
					lPaeth_G = (lPaethA_G <= lPaethB_G && lPaethA_G <= lPaethC_G) ? lCLinPByt_G : (lPaethB_G <= lPaethC_G) ? lPLinCByt_G : lPLinPByt_G;
					udtCLinCByt->xGreen = (BYTE)(((int)(udtCLinCByt->xGreen) + lPaeth_G) & 0xFF);
					
					int lCLinPByt_B, lPLinCByt_B, lPLinPByt_B, lPaethA_B, lPaethB_B, lPaethC_B, lPaeth_B;
					lCLinPByt_B = udtCLinPByt->xBlue;
					lPLinCByt_B = udtPLinCByt->xBlue;
					lPLinPByt_B = udtPLinPByt->xBlue;
					lPaeth_B = lPLinCByt_B - lPLinPByt_B;
					lPaethC_B = lCLinPByt_B - lPLinPByt_B;
					lPaethA_B = lPaeth_B < 0 ? -lPaeth_B : lPaeth_B;
					lPaethB_B = lPaethC_B < 0 ? -lPaethC_B : lPaethC_B;
					lPaethC_B = (lPaeth_B + lPaethC_B) < 0 ? -(lPaeth_B + lPaethC_B) : lPaeth_B + lPaethC_B;
					lPaeth_B = (lPaethA_B <= lPaethB_B && lPaethA_B <= lPaethC_B) ? lCLinPByt_B : (lPaethB_B <= lPaethC_B) ? lPLinCByt_B : lPLinPByt_B;
					udtCLinCByt->xBlue = (BYTE)(((int)(udtCLinCByt->xBlue) + lPaeth_B) & 0xFF);
					
					udtCLinCByt++;
					udtCLinPByt++;
					udtPLinCByt++;
					udtPLinPByt++;
				}
				break;

			/* Unknown filter value */
			default:
				goto GetAOIconData_Exit_Fail;
		}
		
		/* Copy processed row to bitmap (have to do this pixel by pixel because PNG is RGB but DIB is BGR) */
		udtPNGpixel = (udtPNGpixel_struc *)a_xPNGRow;
		rgbDIBpixel = (RGBTRIPLE *)((BYTE *)(bmiDIB->bmiColors) + (((udtPNGihdr->lHeight - 1) - lLoop) * lBytesPerRow));
		for (lLoop2 = 0; lLoop2 < (lBytesPerRow/3); lLoop2++)
		{
			rgbDIBpixel->rgbtRed = udtPNGpixel->xRed;
			rgbDIBpixel->rgbtGreen = udtPNGpixel->xGreen;
			rgbDIBpixel->rgbtBlue = udtPNGpixel->xBlue;
			udtPNGpixel++;
			rgbDIBpixel++;
		}

		/* Copy processed row to previous row buffer */
		memcpy(a_xPNGRowPrev, a_xPNGRow, lBytesPerRow);
	}
	
	/* Release previous row buffer */
	free(a_xPNGRowPrev);
	a_xPNGRowPrev = NULL;

	/* Check IEND chunk - This marks the end of PNG */
	memcpy(strChunkID, (a_xPNGChunk + 0x4), 4);
	strChunkID[4] = 0;
	if (stricmp(strChunkID, "IEND") != 0)
		goto GetAOIconData_Exit_Fail;

	/* Release the PNG image and data chunk */
	free(a_xPNGImage);
	a_xPNGImage = NULL;
	ReleaseDataChunk(&a_xData);

	/* Write icon bitmap to file */
	if (lDebug & DBG_ICN)
	{
		sprintf(strDebugFile, "%sDebug_IconsBMP.DAT", strAOMDPath);
		fpDebug = fopen(strDebugFile, "a+b");
		fwrite(&bmiBMPhdr, sizeof(BITMAPFILEHEADER), 1, fpDebug);
		fwrite(bmiDIB, sizeof(BYTE), lDIBsize, fpDebug);
		fwrite("****************", sizeof(char), 0x10 - ((sizeof(BITMAPFILEHEADER) + lDIBsize) % 0x10), fpDebug);
		fwrite("****************", sizeof(char), 0x10, fpDebug);
		fclose(fpDebug);
	}

	/* Success - return the bitmap */
	return (bmiDIB);
	
GetAOIconData_Exit_Fail:	// Cleanup
	free(bmiDIB);
	free(a_xPNGRowPrev);
	free(a_xPNGImage);
	ReleaseDataChunk(&a_xData);
	return NULL;
}

/* End capture */
void EndCapture(void)
{
	BOOL bResult;
	
	/* Kill the capture thread */
	bResult = TerminateThread(hThread, 0);
	if (!bResult)
		return;
	
	/* Close the adapter */
	pcap_close(fpPcap);

	/* Wait the end of the capture thread */
	WaitForSingleObject(hThread, INFINITE);

	return;
}

/* Give caller the most recent error message */
char *GetErrorMessage(void)
{
	return strError;
}

/* Tokenize the filename if possible */
void TokenizeFilename(char *strDest, char *strSource)
{
	unsigned long lStart;

	/* Initialise */
	lStart = 0;
	strDest[0] = 0;

	/* Test for [AOPath] */
	if (strncmp(strSource, strAOPath, strlen(strAOPath)) == 0)
	{
		lStart = strlen(strAOPath);
		strcpy(strDest, strID_AOPath);
	}

	/* Build tokenized path/file */
	strcat(strDest, (strSource + lStart));
}

/* De-Tokenize the filename if possible */
void DeTokenizeFilename(char *strDest, char *strSource)
{
	unsigned long lStart;

	/* Initialise */
	lStart = 0;
	strDest[0] = 0;

	/* Test for [AOPath] */
	if (strncmp(strSource, strID_AOPath, strlen(strID_AOPath)) == 0)
	{
		lStart = strlen(strID_AOPath);
		strcpy(strDest, strAOPath);
	}

	/* Build full path/file */
	strcat(strDest, (strSource + lStart));
}
