#include <windows.h>
#include <stdio.h>
#include <shlobj.h>
#include <mmsystem.h>
#include <richedit.h>
#include "resource.h"
#include "Main.h"
#include "FirstTimeGUI.h"
#include "AdapterGUI.h"
#include "WatchGUI.h"
#include "OptionsGUI.h"
#include "Functions.h"
#include "AODB.h"

/* Structures */
typedef struct missiontxt_struc
{
	char *strTxt1;
	char *strTxt2;
	char *strTxt3;
	char *strTxt4;
	char *strTxt5;
} udtMissionTxt_struc;
typedef struct icons_struc
{
	BITMAPINFO *bmiIcon1;
	BITMAPINFO *bmiIcon2;
	BITMAPINFO *bmiIcon3;
	BITMAPINFO *bmiIcon4;
	BITMAPINFO *bmiIcon5;
} udtIcons_struc;
typedef struct watchmatch_struc
{
	BOOL bMatch1;
	BOOL bMatch2;
	BOOL bMatch3;
	BOOL bMatch4;
	BOOL bMatch5;
} udtWatchMatch_struc;

/* Function Declarations for internal use */
void CheckCmdLine(void);
unsigned long WINAPI InitWinMM(void *pParam);
void InitSettings(void);
void WriteSettings(void);
ATOM RegisterDummyOwner(HINSTANCE hInstance);
BOOL CreateDummyOwner(HINSTANCE hInstance);
LRESULT CALLBACK DummyOwnerWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
void CloseDummyOwner(void);
void GetFolder(HWND hWndOwner, char *strTitle, char *strPath);
LRESULT CALLBACK MainGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
void GetCtlPosSiz(HWND hWndDlg, HWND hWndCtl, POINT *ptCtlPos, POINT *ptCtlSiz);
void CALLBACK ShowMission(long lMissionIdx, udtMission_struc *udtMission);
BOOL CheckWatchItem(char *strText);
void DrawAOIcon(HWND hWndIcon, BITMAPINFO *bmiIcon);
LRESULT CALLBACK MissionTxtProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

/* Constants */
static char strRichEdDLL[] = "RICHED32.DLL";
static char strAOexe[] = "\\Anarchy.exe";
static char strErrorNoDLL[] = "Failed to load Ctreestd.dll";
static char strErrorNoInit[] = "Failed to initialise Ctree driver";
static char strErrorNoDB[] = "Failed to open AO Database";
static char strRegistryKey[] = "SOFTWARE\\Muzzleflash\\AOMD";
static char strTrue[] = "True";
static char strFalse[] = "False";
static char strSoundFileDefault[] = "[AOPath]\\cd_image\\sound\\sfx\\creatures\\baby\\baby_enterfight_02.wav";	// Wazzup!
static COLORREF cTransparentColor = COL_TRANSPARENT;

/* Variables */
short iMissionTxtIdx;
unsigned long lDebug;
unsigned long lNotifyMethod;
unsigned long lWatchItemsLength;
HINSTANCE hInst;
HWND hWnd;
HWND hWndDlg;
BOOL bFirstTime;
BOOL bMainGUIActive;
BOOL bIconCustomBack;
COLORREF cIconCustomBack;
char strAOMDPathExe[MAX_PATH];
char strAOMDPath[MAX_PATH];
char strAOPath[MAX_PATH];
char strSoundFile[MAX_PATH];
char *strWatchItems;
udtMissionTxt_struc udtMissionTxt;
udtIcons_struc udtIcons;
udtWatchMatch_struc udtWatchMatch;

/* WinMain - The entry point, and main loop */
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int lCmdShow)
{
	long lRetn;
	DWORD dwThreadID;
	HANDLE hFileTest;
	HMODULE hRichEd;
	BOOL bAODBOpen;
	BOOL bCapturing;
	char strAOTest[MAX_PATH];
	char strErrorFmt[255];
	char *strError;
	LPVOID lpErrMsgBuf;
	
	/* Initialise */
	hInst = hInstance;
	lDebug = 0;
	bAODBOpen = false;
	bCapturing = false;
	bMainGUIActive = false;
	
	/* Check command line */
	CheckCmdLine();

	/* Load libraries */
	hRichEd = LoadLibrary(strRichEdDLL);
	if (!hRichEd)
	{
		if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpErrMsgBuf, 0, NULL))
		{
			sprintf(strErrorFmt, "Unable to load \"%s\".\n\n%s", strRichEdDLL, lpErrMsgBuf);
			MessageBox(NULL, strErrorFmt, "Error", MB_ICONERROR);
			LocalFree(lpErrMsgBuf);
		}
		goto WinMain_Exit;
	}

	/* Init the WinMM API */
	CreateThread(NULL, 0, &InitWinMM, NULL, 0, &dwThreadID);

	/* Init functions */
	StartUp();

	/* Fetch settings */
	InitSettings();
	
	/* Create a dummy owner window */
	RegisterDummyOwner(hInstance);
	CreateDummyOwner(hInstance);
	
	/* Show first time dialog if needed */
	if (bFirstTime)
	{
		if (!MainFirstTimeGUI(hInstance, hWnd))
			goto WinMain_Exit;
		bFirstTime = false;
	}

	/* Check/Find AO path */
	do {
		if (!strlen(strAOPath))
			GetFolder(hWnd, "Please select the AO game folder:", strAOPath);
		sprintf(strAOTest, "%s%s", strAOPath, strAOexe);
		hFileTest = CreateFile(strAOTest, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileTest == INVALID_HANDLE_VALUE)
		{
			strAOPath[0] = 0;
			if (MessageBox(hWnd, "Invalid AO path.", "Error", MB_ICONERROR | MB_RETRYCANCEL) == IDCANCEL)
				goto WinMain_Exit;
		}
		else
			CloseHandle(hFileTest);
	} while (!strlen(strAOPath));

	/* Prompt user for adapter to be used */
	if (!MainAdapterGUI(hInstance, hWnd))
		goto WinMain_Exit;

	/* Initialise the AO database */
	lRetn = InitAODatabase(strAOPath);
	if (lRetn)
	{
		switch (lRetn)
		{
			case AODB_ERR_NODLL:
				strError = strErrorNoDLL;
			case AODB_ERR_NOINIT:
				strError = strErrorNoInit;
			case AODB_ERR_NODB:
				strError = strErrorNoDB;
		}
		MessageBox(hWnd, strError, "Error", MB_ICONERROR);
		goto WinMain_Exit;
	}
	bAODBOpen = true;

	/* Start the capture */
	bCapturing = BeginCapture(&ShowMission);
	if (!bCapturing)
	{
		MessageBox(hWnd, GetErrorMessage(), "Error", MB_ICONERROR);
		goto WinMain_Exit;
	}

	/* Display the main GUI */
	ZeroMemory(&udtMissionTxt, sizeof(udtMissionTxt));
	ZeroMemory(&udtIcons, sizeof(udtIcons));
	if (DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAINGUI), hWnd, (DLGPROC)MainGUIProc) < 1)
	{
		if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpErrMsgBuf, 0, NULL))
		{
			MessageBox(NULL, (LPCTSTR)lpErrMsgBuf, "Error", MB_ICONERROR);
			LocalFree(lpErrMsgBuf);
		}
		goto WinMain_Exit;
	}

WinMain_Exit:
	/* Stop the capture */
	if (bCapturing)
	{
		EndCapture();
		bCapturing = false;
	}

	/* Release access to the AO database */
	if (bAODBOpen)
	{
		ReleaseAODatabase();
		bAODBOpen = false;
	}
	
	/* Close the dummy owner window */
	CloseDummyOwner();
	
	/* Write settings */
	WriteSettings();

	/* Free libraries */
	if (hRichEd)
		FreeLibrary(hRichEd);

	return 0;
}

/* Check the command line */
void CheckCmdLine(void)
{
	long lOptionLen;
	char *strCmdLine;
	char *strPosE;
	char strOption[255];

	/* Initialise */
	strAOMDPathExe[0] = 0;
	strAOMDPath[0] = 0;

	/* Set ptr to the command line */
	strCmdLine = GetCommandLine();
	if (!strCmdLine)
		goto CheckCmdLine_Exit;

	/* Store the AOMD executable filename (with full path) */
	strCmdLine = strchr(strCmdLine, 34);	// Find first quote (")
	if (!strCmdLine)
		goto CheckCmdLine_Exit;
	strCmdLine++;
	strPosE = strchr(strCmdLine, 34);		// Find last quote (")
	if (!strPosE)
		goto CheckCmdLine_Exit;
	strncpy(strAOMDPathExe, strCmdLine, (strPosE - strCmdLine));
	strAOMDPathExe[strPosE - strCmdLine] = 0;
	strCmdLine = strPosE + 1;
	
	/* Store AOMD's path */
	strPosE = strrchr(strAOMDPathExe, 92);	// Find last backslash
	if (!strPosE)
		goto CheckCmdLine_Exit;
	strPosE++;
	strncpy(strAOMDPath, strAOMDPathExe, (strPosE - strAOMDPathExe));
	strAOMDPath[strPosE - strAOMDPathExe] = 0;

	/* Look for any command line options */
	while (strlen(strCmdLine))
	{
		/* Bump past any spaces */
		while ((strlen(strCmdLine)) && (*strCmdLine == 32))
			strCmdLine++;
		if (strlen(strCmdLine) < 1)
			break;

		/* Determine next option */
		strPosE = strchr(strCmdLine, 32);
		if (strPosE)
			lOptionLen = strPosE - strCmdLine;
		else
			lOptionLen = strlen(strCmdLine);
		strncpy(strOption, strCmdLine, lOptionLen);
		strOption[lOptionLen] = 0;
		strCmdLine += lOptionLen;

		/* Process option */
		if (stricmp(strOption, "/DEBUG:ALL") == 0)
			lDebug |= DBG_PKT | DBG_MSN | DBG_ITM | DBG_ICN;
		if (stricmp(strOption, "/DEBUG:PKT") == 0)
			lDebug |= DBG_PKT;
		if (stricmp(strOption, "/DEBUG:MSN") == 0)
			lDebug |= DBG_MSN;
		if (stricmp(strOption, "/DEBUG:ITM") == 0)
			lDebug |= DBG_ITM;
		if (stricmp(strOption, "/DEBUG:ICN") == 0)
			lDebug |= DBG_ICN;
	}

CheckCmdLine_Exit:
	return;
}

/* Initialise WinMM API. When playing first sound while AO is running, there is a noticeable delay. To avoid this, we play a dummy sound in another thread when AOMD starts, so that when a real sound is played later, there is no delay */
unsigned long WINAPI InitWinMM(void *pParam)
{
	PlaySound("False name here so nothing is actually played", hInst, SND_RESOURCE | SND_NODEFAULT | SND_ASYNC);
	
	ExitThread(0);
	return 0;
}

/* Initialise program settings from registry */
void InitSettings(void)
{
	unsigned long lLength;
	HKEY hRegKey;
	char strBool[10];
	
	/* Open registry */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, strRegistryKey, 0, KEY_ALL_ACCESS, &hRegKey) != ERROR_SUCCESS)
	{
		bFirstTime = true;
		strAOPath[0] = 0;
		bIconCustomBack = false;
		cIconCustomBack = 0;
		lNotifyMethod = NFY_SOUND;
		strcpy(strSoundFile, strSoundFileDefault);
		lWatchItemsLength = 2;
		strWatchItems = (char *)malloc(lWatchItemsLength);
		strWatchItems[0] = 0;
		strWatchItems[1] = 0;
		return;
	}

	/* Fetch 'First Time' flag */
	lLength = sizeof(strBool);
	if ((RegQueryValueEx(hRegKey, "FirstTime", NULL, NULL, (BYTE *)strBool, &lLength) != ERROR_SUCCESS) || (lLength < sizeof(strTrue)) || (lLength > sizeof(strFalse)))
		memcpy(strBool, strTrue, sizeof(strTrue));	// Init to True
	if (stricmp(strBool, strTrue) == 0)
		bFirstTime = true;
	else
		bFirstTime = false;

	/* Fetch AO path */
	lLength = sizeof(strAOPath);
	if (RegQueryValueEx(hRegKey, "AOPath", NULL, NULL, (BYTE *)strAOPath, &lLength) != ERROR_SUCCESS)
		strAOPath[0] = 0;	// Init to nothing

	/* Fetch icon custom background flag */
	lLength = sizeof(strBool);
	if ((RegQueryValueEx(hRegKey, "IconCustomBackground", NULL, NULL, (BYTE *)strBool, &lLength) != ERROR_SUCCESS) || (lLength < sizeof(strTrue)) || (lLength > sizeof(strFalse)))
		memcpy(strBool, strFalse, sizeof(strFalse));	// Init to False
	if (stricmp(strBool, strTrue) == 0)
		bIconCustomBack = true;
	else
		bIconCustomBack = false;

	/* Fetch icon custom background colour */
	lLength = sizeof(COLORREF);
	if (RegQueryValueEx(hRegKey, "IconCustomBackgroundColour", NULL, NULL, (BYTE *)&cIconCustomBack, &lLength) != ERROR_SUCCESS)
		cIconCustomBack = 0;	// Init to Black

	/* Fetch item watch notify method */
	lLength = sizeof(unsigned long);
	if (RegQueryValueEx(hRegKey, "NotifyMethod", NULL, NULL, (BYTE *)&lNotifyMethod, &lLength) != ERROR_SUCCESS)
		lNotifyMethod = NFY_SOUND;		// Init to Sound

	/* Fetch notify sound filename */
	lLength = sizeof(strSoundFile);
	if (RegQueryValueEx(hRegKey, "NotifySoundFile", NULL, NULL, (BYTE *)strSoundFile, &lLength) != ERROR_SUCCESS)
		strcpy(strSoundFile, strSoundFileDefault);

	/* Fetch items to watch for */
	if (RegQueryValueEx(hRegKey, "WatchItems", NULL, NULL, NULL, &lWatchItemsLength) != ERROR_SUCCESS)	// Queries size of value only
	{
		lWatchItemsLength = 2;
		strWatchItems = (char *)malloc(lWatchItemsLength);
		strWatchItems[0] = 0;
		strWatchItems[1] = 0;
	}
	else
	{
		strWatchItems = (char *)malloc(lWatchItemsLength);
		if (RegQueryValueEx(hRegKey, "WatchItems", NULL, NULL, (BYTE *)strWatchItems, &lWatchItemsLength) != ERROR_SUCCESS)
		{
			free(strWatchItems);
			lWatchItemsLength = 2;
			strWatchItems = (char *)malloc(lWatchItemsLength);
			strWatchItems[0] = 0;
			strWatchItems[1] = 0;
		}
	}

	/* Close registry */
	RegCloseKey(hRegKey);
}

/* Write program settings to the registry */
void WriteSettings(void)
{
	HKEY hRegKey;
	char *strBool;
	
	/* Create registry key */
	if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, strRegistryKey, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hRegKey, NULL) != ERROR_SUCCESS)
		return;

	/* Write AOMD executable location */
	RegSetValueEx(hRegKey, NULL, 0, REG_SZ, (BYTE *)strAOMDPathExe, strlen(strAOMDPathExe)+1);

	/* Write 'First Time' flag */
	if (bFirstTime)
		strBool = strTrue;
	else
		strBool = strFalse;
	RegSetValueEx(hRegKey, "FirstTime", 0, REG_SZ, (BYTE *)strBool, strlen(strBool)+1);

	/* Write AO path */
	RegSetValueEx(hRegKey, "AOPath", 0, REG_SZ, (BYTE *)strAOPath, strlen(strAOPath)+1);

	/* Write icon custom background flag */
	if (bIconCustomBack)
		strBool = strTrue;
	else
		strBool = strFalse;
	RegSetValueEx(hRegKey, "IconCustomBackground", 0, REG_SZ, (BYTE *)strBool, strlen(strBool)+1);

	/* Write icon custom background colour */
	RegSetValueEx(hRegKey, "IconCustomBackgroundColour", 0, REG_DWORD, (BYTE *)&cIconCustomBack, sizeof(COLORREF));

	/* Write item watch notify method */
	RegSetValueEx(hRegKey, "NotifyMethod", 0, REG_DWORD, (BYTE *)&lNotifyMethod, sizeof(unsigned long));

	/* Write notify sound filename */
	RegSetValueEx(hRegKey, "NotifySoundFile", 0, REG_SZ, (BYTE *)strSoundFile, strlen(strSoundFile)+1);

	/* Write items to watch for */
	RegSetValueEx(hRegKey, "WatchItems", 0, REG_MULTI_SZ, (BYTE *)strWatchItems, lWatchItemsLength);
	free(strWatchItems);

	/* Close registry */
	RegCloseKey(hRegKey);
}

/* Registers the dummy owner class. */
ATOM RegisterDummyOwner(HINSTANCE hInstance)
{
	WNDCLASSEX udtDummyOwnerWCEX;

	udtDummyOwnerWCEX.cbSize = sizeof(WNDCLASSEX); 
	udtDummyOwnerWCEX.style = CS_HREDRAW | CS_VREDRAW;
	udtDummyOwnerWCEX.lpfnWndProc = DummyOwnerWndProc;
	udtDummyOwnerWCEX.cbClsExtra = 0;
	udtDummyOwnerWCEX.cbWndExtra = 0;
	udtDummyOwnerWCEX.hInstance = hInstance;
	udtDummyOwnerWCEX.hIcon = LoadIcon(hInstance, (LPCTSTR)IDI_AOMD);
	udtDummyOwnerWCEX.hCursor = LoadCursor(NULL, IDC_ARROW);
	udtDummyOwnerWCEX.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	udtDummyOwnerWCEX.lpszMenuName = NULL;
	udtDummyOwnerWCEX.lpszClassName = "AOMD";
	udtDummyOwnerWCEX.hIconSm = LoadIcon(hInstance, (LPCTSTR)IDI_AOMD);

	return RegisterClassEx(&udtDummyOwnerWCEX);
}

/* Creates dummy owner window */
BOOL CreateDummyOwner(HINSTANCE hInstance)
{
	/* Create window */
	hWnd = CreateWindowEx(WS_EX_APPWINDOW, "AOMD", "AO Mission Display", WS_OVERLAPPED, CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);
	if (!hWnd)
		return false;

	/* Show window, minimized, never to be seen */
	ShowWindow(hWnd, SW_MINIMIZE);

	return true;
}

/* Dummy owner window message handler */
LRESULT CALLBACK DummyOwnerWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return (DefWindowProc(hWnd, msg, wParam, lParam));
	}

	return 0;
}

/* Closes dummy owner window */
void CloseDummyOwner(void)
{
	DestroyWindow(hWnd);
}

/* Prompt user for folder */
void GetFolder(HWND hWndOwner, char *strTitle, char *strPath)
{
	BROWSEINFO udtBI;
	ITEMIDLIST *udtIDList;

	/* Initialise */
	udtBI.hwndOwner = hWndOwner;
	udtBI.pidlRoot = NULL;
	udtBI.pszDisplayName = NULL;
	udtBI.lpszTitle = strTitle;
	udtBI.ulFlags = BIF_RETURNONLYFSDIRS;
	udtBI.lpfn = NULL;
	udtBI.lParam = NULL;
	udtBI.iImage = 0;

	/* Prompt user for folder */
	udtIDList = SHBrowseForFolder(&udtBI);
	
	/* Extract pathname */
	if (!SHGetPathFromIDList(udtIDList, strPath))
		strAOPath[0] = 0;	// Zero-length if failure
}

/* Window message handler */
LRESULT CALLBACK MainGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	long lWidth;
	long lHeight;
	static long lOldWidth;
	long lWDiff;
	HWND hCtl;
	HDC hDCDlg;
	PAINTSTRUCT psDlg;
	static POINT ptMin;
	static POINT ptOld;
	POINT ptCtlPos;
	POINT ptCtlSiz;
	RECT udtRect;
	MINMAXINFO *udtMinMax;

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			/* Save dialog handle */
			hWndDlg = hDlg;

			/* Store width/height (used as minimum limit) */
			GetWindowRect(hDlg, &udtRect);
			ptMin.x = udtRect.right - udtRect.left;
			ptMin.y = udtRect.bottom - udtRect.top;
			
			/* Init 'old' dialog width value */
			GetClientRect(hDlg, &udtRect);
			lOldWidth = udtRect.right - udtRect.left;

			/* Set to bottom right corner */
			GetClientRect(GetDesktopWindow(), &udtRect);
			MoveWindow(hDlg, udtRect.right - ptMin.x, udtRect.bottom - ptMin.y, ptMin.x, ptMin.y, false);

			/* Tell other thread we're ready to take updates */
			bMainGUIActive = true;

			return true;

		/* Paints the window */
		case WM_PAINT:
			hDCDlg = BeginPaint(hDlg, &psDlg);

			/* Draw icons */
			DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon1), udtIcons.bmiIcon1);
			DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon2), udtIcons.bmiIcon2);
			DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon3), udtIcons.bmiIcon3);
			DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon4), udtIcons.bmiIcon4);
			DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon5), udtIcons.bmiIcon5);

			EndPaint(hDlg, &psDlg);
  			return false;

		/* Restrict min/max settings */
		case WM_GETMINMAXINFO:
			udtMinMax = (MINMAXINFO *)lParam;
			udtMinMax->ptMinTrackSize.x = ptMin.x;
			udtMinMax->ptMinTrackSize.y = ptMin.y;
			udtMinMax->ptMaxTrackSize.y = ptMin.y;
			return false;

		/* Resizing window */
		case WM_SIZE:
			switch (wParam)
			{
				case SIZE_MAXIMIZED:
				case SIZE_RESTORED:
					lWidth = LOWORD(lParam);
					lHeight = HIWORD(lParam);

					/* Calculate width change */
					lWDiff = lWidth - lOldWidth;
					lOldWidth = lWidth;
					
					/* Mission 1 controls */
					hCtl = GetDlgItem(hDlg, IDC_frmM1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMLoc1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_lblMCash1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMCash1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMItem1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMQL1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMFind1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_picMIcon1);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					
					/* Mission 2 controls */
					hCtl = GetDlgItem(hDlg, IDC_frmM2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMLoc2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_lblMCash2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMCash2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMItem2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMQL2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMFind2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_picMIcon2);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					
					/* Mission 3 controls */
					hCtl = GetDlgItem(hDlg, IDC_frmM3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMLoc3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_lblMCash3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMCash3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMItem3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMQL3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMFind3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_picMIcon3);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					
					/* Mission 4 controls */
					hCtl = GetDlgItem(hDlg, IDC_frmM4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMLoc4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_lblMCash4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMCash4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMItem4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMQL4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMFind4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_picMIcon4);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					
					/* Mission 5 controls */
					hCtl = GetDlgItem(hDlg, IDC_frmM5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMLoc5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_lblMCash5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMCash5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMItem5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMQL5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_txtMFind5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, ptCtlPos.x, ptCtlPos.y, (ptCtlSiz.x + lWDiff), ptCtlSiz.y, false);
					hCtl = GetDlgItem(hDlg, IDC_picMIcon5);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);
					
					/* Options button */
					hCtl = GetDlgItem(hDlg, IDC_cmdOptions);
					GetCtlPosSiz(hDlg, hCtl, &ptCtlPos, &ptCtlSiz);
					MoveWindow(hCtl, (ptCtlPos.x + lWDiff), ptCtlPos.y, ptCtlSiz.x, ptCtlSiz.y, false);

					/* Refresh window */
					InvalidateRect(hDlg, NULL, true);
					UpdateWindow(hDlg);

					/* Force to top window */
					SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

					return false;
			}
			break;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_cmdMDesc1:
					iMissionTxtIdx = 0;
					if (udtMissionTxt.strTxt1)
						DialogBox(hInst, MAKEINTRESOURCE(IDD_MISSIONTXT), hDlg, (DLGPROC)MissionTxtProc);
					return true;
				case IDC_cmdMDesc2:
					iMissionTxtIdx = 1;
					if (udtMissionTxt.strTxt2)
						DialogBox(hInst, MAKEINTRESOURCE(IDD_MISSIONTXT), hDlg, (DLGPROC)MissionTxtProc);
					return true;
				case IDC_cmdMDesc3:
					iMissionTxtIdx = 2;
					if (udtMissionTxt.strTxt3)
						DialogBox(hInst, MAKEINTRESOURCE(IDD_MISSIONTXT), hDlg, (DLGPROC)MissionTxtProc);
					return true;
				case IDC_cmdMDesc4:
					iMissionTxtIdx = 3;
					if (udtMissionTxt.strTxt4)
						DialogBox(hInst, MAKEINTRESOURCE(IDD_MISSIONTXT), hDlg, (DLGPROC)MissionTxtProc);
					return true;
				case IDC_cmdMDesc5:
					iMissionTxtIdx = 4;
					if (udtMissionTxt.strTxt5)
						DialogBox(hInst, MAKEINTRESOURCE(IDD_MISSIONTXT), hDlg, (DLGPROC)MissionTxtProc);
					return true;
				case IDC_cmdWatch:
					WatchGUI(hInst, hWndDlg);
					return true;
				case IDC_cmdOptions:
					if (OptionsGUI(hInst, hWndDlg))
					{
						/* Redraw icons */
						DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon1), udtIcons.bmiIcon1);
						DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon2), udtIcons.bmiIcon2);
						DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon3), udtIcons.bmiIcon3);
						DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon4), udtIcons.bmiIcon4);
						DrawAOIcon(GetDlgItem(hDlg, IDC_picMIcon5), udtIcons.bmiIcon5);
					}
					return true;
			}
			break;

		/* Process commands */
		case WM_SYSCOMMAND:
			switch (wParam)
			{
				case SC_CLOSE:
					EndDialog(hDlg, 1);
					return false;
			}
			break;

		/* Window destroyed */
		case WM_DESTROY:
			/* Ensure other thread doesn't keep updating */
			bMainGUIActive = false;

			/* Release mission text */
			free(udtMissionTxt.strTxt1);
			free(udtMissionTxt.strTxt2);
			free(udtMissionTxt.strTxt3);
			free(udtMissionTxt.strTxt4);
			free(udtMissionTxt.strTxt5);

			/* Release item icons */
			free(udtIcons.bmiIcon1);
			free(udtIcons.bmiIcon2);
			free(udtIcons.bmiIcon3);
			free(udtIcons.bmiIcon4);
			free(udtIcons.bmiIcon5);

			return false;
	}

	return false;
}

/* Get position and size of a control */
void GetCtlPosSiz(HWND hWndDlg, HWND hWndCtl, POINT *ptCtlPos, POINT *ptCtlSiz)
{
	RECT udtRectAdj;
	RECT udtRectDlg;
	RECT udtRectCtl;

	/* Determine adjustments */
	udtRectAdj.top = 0;
	udtRectAdj.left = 0;
	udtRectAdj.bottom = 0;
	udtRectAdj.right = 0;
	AdjustWindowRectEx(&udtRectAdj, WS_CAPTION, false, WS_EX_TOOLWINDOW);

	/* Calculate control position */
	GetWindowRect(hWndDlg, &udtRectDlg);
	GetWindowRect(hWndCtl, &udtRectCtl);
	ptCtlPos->x = (udtRectCtl.left - udtRectDlg.left) + udtRectAdj.left - 1;
	ptCtlPos->y = (udtRectCtl.top - udtRectDlg.top) + udtRectAdj.top - 1;

	/* Calculate control size */
	GetWindowRect(hWndCtl, &udtRectCtl);
	ptCtlSiz->x = udtRectCtl.right - udtRectCtl.left;
	ptCtlSiz->y = udtRectCtl.bottom - udtRectCtl.top;
}

/* Show mission details */
void CALLBACK ShowMission(long lMissionIdx, udtMission_struc *udtMission)
{
	HWND hWnd_txtMLoc;
	HWND hWnd_txtMCash;
	HWND hWnd_txtMItem;
	HWND hWnd_txtMQL;
	HWND hWnd_txtMFind;
	HWND hWnd_picMIcon;
	BOOL bMatch;
	char strCash[20];
	char strQL[20];
	char *strMissionTxt;
	char strFile[MAX_PATH];
	BITMAPINFO *bmiIcon;
	BITMAPINFO *bmiOldIcon;

	/* Ensure the main GUI is active before continuing */
	if (!bMainGUIActive)
	{
		if (udtMission)
			free(udtMission->bmiIcon);
		goto ShowMission_Exit;
	}

	/* Determine control handles */
	switch (lMissionIdx)
	{
		case 0:
			hWnd_txtMLoc = GetDlgItem(hWndDlg, IDC_txtMLoc1);
			hWnd_txtMCash = GetDlgItem(hWndDlg, IDC_txtMCash1);
			hWnd_txtMItem = GetDlgItem(hWndDlg, IDC_txtMItem1);
			hWnd_txtMQL = GetDlgItem(hWndDlg, IDC_txtMQL1);
			hWnd_txtMFind = GetDlgItem(hWndDlg, IDC_txtMFind1);
			hWnd_picMIcon = GetDlgItem(hWndDlg, IDC_picMIcon1);
			break;
		case 1:
			hWnd_txtMLoc = GetDlgItem(hWndDlg, IDC_txtMLoc2);
			hWnd_txtMCash = GetDlgItem(hWndDlg, IDC_txtMCash2);
			hWnd_txtMItem = GetDlgItem(hWndDlg, IDC_txtMItem2);
			hWnd_txtMQL = GetDlgItem(hWndDlg, IDC_txtMQL2);
			hWnd_txtMFind = GetDlgItem(hWndDlg, IDC_txtMFind2);
			hWnd_picMIcon = GetDlgItem(hWndDlg, IDC_picMIcon2);
			break;
		case 2:
			hWnd_txtMLoc = GetDlgItem(hWndDlg, IDC_txtMLoc3);
			hWnd_txtMCash = GetDlgItem(hWndDlg, IDC_txtMCash3);
			hWnd_txtMItem = GetDlgItem(hWndDlg, IDC_txtMItem3);
			hWnd_txtMQL = GetDlgItem(hWndDlg, IDC_txtMQL3);
			hWnd_txtMFind = GetDlgItem(hWndDlg, IDC_txtMFind3);
			hWnd_picMIcon = GetDlgItem(hWndDlg, IDC_picMIcon3);
			break;
		case 3:
			hWnd_txtMLoc = GetDlgItem(hWndDlg, IDC_txtMLoc4);
			hWnd_txtMCash = GetDlgItem(hWndDlg, IDC_txtMCash4);
			hWnd_txtMItem = GetDlgItem(hWndDlg, IDC_txtMItem4);
			hWnd_txtMQL = GetDlgItem(hWndDlg, IDC_txtMQL4);
			hWnd_txtMFind = GetDlgItem(hWndDlg, IDC_txtMFind4);
			hWnd_picMIcon = GetDlgItem(hWndDlg, IDC_picMIcon4);
			break;
		case 4:
			hWnd_txtMLoc = GetDlgItem(hWndDlg, IDC_txtMLoc5);
			hWnd_txtMCash = GetDlgItem(hWndDlg, IDC_txtMCash5);
			hWnd_txtMItem = GetDlgItem(hWndDlg, IDC_txtMItem5);
			hWnd_txtMQL = GetDlgItem(hWndDlg, IDC_txtMQL5);
			hWnd_txtMFind = GetDlgItem(hWndDlg, IDC_txtMFind5);
			hWnd_picMIcon = GetDlgItem(hWndDlg, IDC_picMIcon5);
			break;
		default:	// Invalid mission number - shouldn't happen
			goto ShowMission_Exit;
	}

	/* If mission exists, then display details */
	if (udtMission)
	{
		/* Convert cash and item QL to strings for display */
		sprintf(strCash, "%d", udtMission->lCash);
		sprintf(strQL, "QL%d", udtMission->iQL);

		/* Set control text */
		SendMessage(hWnd_txtMLoc, WM_SETTEXT, 0, (LPARAM)udtMission->strPFName);
		SendMessage(hWnd_txtMCash, WM_SETTEXT, 0, (LPARAM)strCash);
		SendMessage(hWnd_txtMItem, WM_SETTEXT, 0, (LPARAM)udtMission->strItemName);
		SendMessage(hWnd_txtMQL, WM_SETTEXT, 0, (LPARAM)strQL);
		if (udtMission->strFindName)
			SendMessage(hWnd_txtMFind, WM_SETTEXT, 0, (LPARAM)udtMission->strFindName);
		else
			SendMessage(hWnd_txtMFind, WM_SETTEXT, 0, (LPARAM)"");
	}
	/* Else mission doesn't exist, so blank display */
	else
	{
		/* Set control text */
		SendMessage(hWnd_txtMLoc, WM_SETTEXT, 0, (LPARAM)"");
		SendMessage(hWnd_txtMCash, WM_SETTEXT, 0, (LPARAM)"");
		SendMessage(hWnd_txtMItem, WM_SETTEXT, 0, (LPARAM)"");
		SendMessage(hWnd_txtMQL, WM_SETTEXT, 0, (LPARAM)"");
		SendMessage(hWnd_txtMFind, WM_SETTEXT, 0, (LPARAM)"");
	}

	/* Update mission text array */
	if (udtMission)
	{
		strMissionTxt = (char *)malloc(udtMission->lDescLen);
		memcpy(strMissionTxt, udtMission->strDesc, udtMission->lDescLen);
	}
	else
		strMissionTxt = NULL;
	switch (lMissionIdx)
	{
		case 0:
			free(udtMissionTxt.strTxt1);
			udtMissionTxt.strTxt1 = strMissionTxt;
			break;
		case 1:
			free(udtMissionTxt.strTxt2);
			udtMissionTxt.strTxt2 = strMissionTxt;
			break;
		case 2:
			free(udtMissionTxt.strTxt3);
			udtMissionTxt.strTxt3 = strMissionTxt;
			break;
		case 3:
			free(udtMissionTxt.strTxt4);
			udtMissionTxt.strTxt4 = strMissionTxt;
			break;
		case 4:
			free(udtMissionTxt.strTxt5);
			udtMissionTxt.strTxt5 = strMissionTxt;
			break;
	}

	/* Update icon array */
	if (udtMission)
		bmiIcon = udtMission->bmiIcon;
	else
		bmiIcon = NULL;
	switch (lMissionIdx)
	{
		case 0:
			bmiOldIcon = udtIcons.bmiIcon1;
			udtIcons.bmiIcon1 = bmiIcon;
			break;
		case 1:
			bmiOldIcon = udtIcons.bmiIcon2;
			udtIcons.bmiIcon2 = bmiIcon;
			break;
		case 2:
			bmiOldIcon = udtIcons.bmiIcon3;
			udtIcons.bmiIcon3 = bmiIcon;
			break;
		case 3:
			bmiOldIcon = udtIcons.bmiIcon4;
			udtIcons.bmiIcon4 = bmiIcon;
			break;
		case 4:
			bmiOldIcon = udtIcons.bmiIcon5;
			udtIcons.bmiIcon5 = bmiIcon;
			break;
	}

	/* Draw the new icon */
	DrawAOIcon(hWnd_picMIcon, bmiIcon);

	/* Release the old icon */
	free(bmiOldIcon);

	/* Check for watch items */
	if (udtMission)
	{
		bMatch = CheckWatchItem(udtMission->strItemName);
		if (!bMatch && (udtMission->strFindName))
			bMatch = CheckWatchItem(udtMission->strFindName);
	}
	else
		bMatch = false;

	/* Update watch match index */
	switch (lMissionIdx)
	{
		case 0:
			udtWatchMatch.bMatch1 = bMatch;
			break;
		case 1:
			udtWatchMatch.bMatch2 = bMatch;
			break;
		case 2:
			udtWatchMatch.bMatch3 = bMatch;
			break;
		case 3:
			udtWatchMatch.bMatch4 = bMatch;
			break;
		case 4:
			udtWatchMatch.bMatch5 = bMatch;
			break;
	}

	/* If the last mission and there was an item watch match, issue notification */
	if ((lMissionIdx == (AO_MISSION_MAX-1)) && (udtWatchMatch.bMatch1 | udtWatchMatch.bMatch2 | udtWatchMatch.bMatch3 | udtWatchMatch.bMatch4 | udtWatchMatch.bMatch5))
	{
		switch (lNotifyMethod)
		{
			/* Display message */
			case NFY_VISUAL:
				MessageBox(hWndDlg, "One of the item rewards matches your search parameters", "Item Watch Match", MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
				break;
			
			/* Play sound */
			case NFY_SOUND:
				DeTokenizeFilename(strFile, strSoundFile);
				PlaySound(strFile, NULL, SND_FILENAME | SND_NODEFAULT | SND_ASYNC);
				break;
		}
	}

ShowMission_Exit:
	return;
}

/* Check text to see if it includes one of the watch strings */
BOOL CheckWatchItem(char *strText)
{
	long lLoop;
	long lMax;
	char *strItem;

	for (strItem = strWatchItems; strItem[0] != 0; strItem += (strlen(strItem) + 1))
	{
		lMax = strlen(strText) - strlen(strItem);
		for (lLoop = 0; lLoop <= lMax; lLoop++)
			if (strnicmp(strItem, (strText + lLoop), strlen(strItem)) == 0)
				return true;
	}

	return false;
}

/* Draw the AO Icon */
void DrawAOIcon(HWND hWndIcon, BITMAPINFO *bmiIcon)
{
	COLORREF cColor;
	HBRUSH brBackground;
	BITMAP bmTemp;
	HBITMAP bmIcon;
	HBITMAP bmAndBack, bmAndObject, bmAndMem, bmSave;
	HBITMAP bmBackOld, bmObjectOld, bmMemOld, bmSaveOld;
	HDC hDCIcon;
	HDC hDCMem, hDCBack, hDCObject, hDCTemp, hDCSave;

	/* Get device context for icon */
	hDCIcon = GetDC(hWndIcon);

	/* Create coloured background */
	hDCMem = CreateCompatibleDC(hDCIcon);
	bmIcon = (HBITMAP)LoadImage(hInst, MAKEINTRESOURCE(IDB_DUMMYICON), IMAGE_BITMAP, 0, 0, LR_SHARED);
	bmIcon = (HBITMAP)SelectObject(hDCMem, bmIcon);
	if (bIconCustomBack)	// Use custom background colour
		brBackground = CreateSolidBrush(cIconCustomBack);
	else					// Use frame colour (makes icon look transparent)
		brBackground = GetSysColorBrush(GetSysColor(COLOR_WINDOWFRAME));
	brBackground = (HBRUSH)SelectObject(hDCMem, brBackground);
	ExtFloodFill(hDCMem, 0, 0, cTransparentColor, FLOODFILLSURFACE);
	BitBlt(hDCIcon, 0, 0, 48, 48, hDCMem, 0, 0, SRCCOPY);
	DeleteObject(SelectObject(hDCMem, bmIcon));
	DeleteObject(SelectObject(hDCMem, brBackground));
	DeleteDC(hDCMem);

	/* If icon was provided, draw transparently onto background */
	if (bmiIcon)
	{
		/* Get bitmap to display */
		bmIcon = CreateCompatibleBitmap(hDCIcon, 48, 48);
		SetDIBits(hDCIcon, bmIcon, 0, 48, (BYTE *)bmiIcon + sizeof(BITMAPINFOHEADER), bmiIcon, DIB_RGB_COLORS);

		/* Allocate temporary device context to hold bitmap */
		hDCTemp = CreateCompatibleDC(hDCIcon);
		SelectObject(hDCTemp, bmIcon);
		GetObject(bmIcon, sizeof(BITMAP), (LPSTR)&bmTemp);

		/* Create some DCs to hold temporary data */
		hDCBack = CreateCompatibleDC(hDCIcon);
		hDCObject = CreateCompatibleDC(hDCIcon);
		hDCMem = CreateCompatibleDC(hDCIcon);
		hDCSave = CreateCompatibleDC(hDCIcon);

		/* Monochrome DCs */
		bmAndBack = CreateBitmap(48, 48, 1, 1, NULL);
		bmAndObject = CreateBitmap(48, 48, 1, 1, NULL);

		bmAndMem = CreateCompatibleBitmap(hDCIcon, 48, 48);
		bmSave = CreateCompatibleBitmap(hDCIcon, 48, 48);

		/* Each DC must select a bitmap object to store pixel data */
		bmBackOld = (HBITMAP)SelectObject(hDCBack, bmAndBack);
		bmObjectOld = (HBITMAP)SelectObject(hDCObject, bmAndObject);
		bmMemOld = (HBITMAP)SelectObject(hDCMem, bmAndMem);
		bmSaveOld = (HBITMAP)SelectObject(hDCSave, bmSave);

		/* Set proper mapping mode */
		SetMapMode(hDCTemp, GetMapMode(hDCIcon));

		/* Save the bitmap sent here, because it will be overwritten */
		BitBlt(hDCSave, 0, 0, 48, 48, hDCTemp, 0, 0, SRCCOPY);

		/* Set the background color of the source DC to the color.contained in the parts of the bitmap that should be transparent */
		cColor = SetBkColor(hDCTemp, cTransparentColor);

		/* Create the object mask for the bitmap by performing a BitBlt from the source bitmap to a monochrome bitmap */
		BitBlt(hDCObject, 0, 0, 48, 48, hDCTemp, 0, 0, SRCCOPY);

		/* Set the background color of the source DC back to the original color */
		SetBkColor(hDCTemp, cColor);

		/* Create the inverse of the object mask */
		BitBlt(hDCBack, 0, 0, 48, 48, hDCObject, 0, 0, NOTSRCCOPY);

		/* Copy the background of the main DC to the destination */
		BitBlt(hDCMem, 0, 0, 48, 48, hDCIcon, 0, 0, SRCCOPY);

		/* Mask out the places where the bitmap will be placed */
		BitBlt(hDCMem, 0, 0, 48, 48, hDCObject, 0, 0, SRCAND);

		/* Mask out the transparent colored pixels on the bitmap */
		BitBlt(hDCTemp, 0, 0, 48, 48, hDCBack, 0, 0, SRCAND);

		/* XOR the bitmap with the background on the destination DC */
		BitBlt(hDCMem, 0, 0, 48, 48, hDCTemp, 0, 0, SRCPAINT);

		/* Copy the destination to the screen */
		BitBlt(hDCIcon, 0, 0, 48, 48, hDCMem, 0, 0, SRCCOPY);

		/* Place the original bitmap back into the bitmap sent here */
		BitBlt(hDCTemp, 0, 0, 48, 48, hDCSave, 0, 0, SRCCOPY);

		/* Delete the memory bitmaps */
		DeleteObject(SelectObject(hDCBack, bmBackOld));
		DeleteObject(SelectObject(hDCObject, bmObjectOld));
		DeleteObject(SelectObject(hDCMem, bmMemOld));
		DeleteObject(SelectObject(hDCSave, bmSaveOld));

		/* Delete the memory DCs */
		DeleteDC(hDCMem);
		DeleteDC(hDCBack);
		DeleteDC(hDCObject);
		DeleteDC(hDCSave);
		DeleteDC(hDCTemp);
	}

	/* Release icon's device context */
	ReleaseDC(hWndIcon, hDCIcon);
}

/* Window message handler */
LRESULT CALLBACK MissionTxtProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hWnd_txtMissionText;

	/* Init variables */
	hWnd_txtMissionText = GetDlgItem(hDlg, IDC_txtMissionText);

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			/* Set mission description text */
			switch (iMissionTxtIdx)
			{
				case 0:
					SendMessage(hWnd_txtMissionText, WM_SETTEXT, 0, (LPARAM)udtMissionTxt.strTxt1);
					break;
				case 1:
					SendMessage(hWnd_txtMissionText, WM_SETTEXT, 0, (LPARAM)udtMissionTxt.strTxt2);
					break;
				case 2:
					SendMessage(hWnd_txtMissionText, WM_SETTEXT, 0, (LPARAM)udtMissionTxt.strTxt3);
					break;
				case 3:
					SendMessage(hWnd_txtMissionText, WM_SETTEXT, 0, (LPARAM)udtMissionTxt.strTxt4);
					break;
				case 4:
					SendMessage(hWnd_txtMissionText, WM_SETTEXT, 0, (LPARAM)udtMissionTxt.strTxt5);
					break;
			}
			
			return true;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_txtMissionText:
					if (HIWORD(wParam) == EN_SETFOCUS)
					{
						SendMessage(hWnd_txtMissionText, EM_SETSEL, 0, 0);
						return false;
					}
					break;
			}
			break;

		/* Process commands */
		case WM_SYSCOMMAND:
			switch (wParam)
			{
				case SC_CLOSE:
					EndDialog(hDlg, 1);
					return false;
			}
			break;
	}

	return false;
}
