#include <windows.h>
#include <mmsystem.h>
#include "resource.h"
#include "Main.h"
#include "OptionsGUI.h"
#include "Functions.h"

/* Function Declarations for internal use */
LRESULT CALLBACK OptionsGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
char *CompactFilename(HWND hWndObject, char *strFilename, RECT *recDisplay);
void DrawCustomIconBack(HWND hWndBack);

/* Constants */
static COLORREF cTransparentColor = COL_TRANSPARENT;

/* Variables */
unsigned long lNotifyMethodOpt;
BOOL bIconCustomBackOpt;
COLORREF cIconCustomBackOpt;
COLORREF a_cCustomColors[0x10];
HINSTANCE hInstOpt;
char strSoundFileOpt[MAX_PATH];

/* The main routine */
BOOL OptionsGUI(HINSTANCE hInstance, HWND hWndOwner)
{
	/* Save options to local variables */
	hInstOpt = hInstance;
	bIconCustomBackOpt = bIconCustomBack;
	cIconCustomBackOpt = cIconCustomBack;
	lNotifyMethodOpt = lNotifyMethod;
	strcpy(strSoundFileOpt, strSoundFile);

	/* Show the options dialog */
	if (!DialogBox(hInstance, MAKEINTRESOURCE(IDD_OPTIONSGUI), hWndOwner, (DLGPROC)OptionsGUIProc))
		return false;

	/* User clicked OK, return updated options */
	bIconCustomBack = bIconCustomBackOpt;
	cIconCustomBack = cIconCustomBackOpt;
	lNotifyMethod = lNotifyMethodOpt;
	strcpy(strSoundFile, strSoundFileOpt);
	return true;
}

/* Window message handler */
LRESULT CALLBACK OptionsGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	unsigned short iFileExtIndex;
	HWND hWnd_picMIconOpt;
	HWND hWnd_cmdSoundFile;
	HWND hWnd_cmdPlay;
	HDC hDCDlg;
	RECT recWin, recCli;
	static HBITMAP bmPlay;
	char strFile[MAX_PATH];
	char strInitialDir[MAX_PATH];
	char *strPos;
	PAINTSTRUCT psDlg;
	CHOOSECOLOR ccIconCustom;
	OPENFILENAME ofnSoundFile;

	/* Init variables */
	hWnd_picMIconOpt = GetDlgItem(hDlg, IDC_picMIconOpt);
	hWnd_cmdSoundFile = GetDlgItem(hDlg, IDC_cmdSoundFile);
	hWnd_cmdPlay = GetDlgItem(hDlg, IDC_cmdPlay);

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			/* Set the correct icon background option button */
			if (!bIconCustomBackOpt)
				CheckRadioButton(hDlg, IDC_btnIcon1, IDC_btnIcon2, IDC_btnIcon1);
			else
				CheckRadioButton(hDlg, IDC_btnIcon1, IDC_btnIcon2, IDC_btnIcon2);

			/* Set the icon background size */
			GetWindowRect(hWnd_picMIconOpt, &recWin);
			GetClientRect(hWnd_picMIconOpt, &recCli);
			SetWindowPos(hWnd_picMIconOpt, NULL, 0, 0, 48 + ((recWin.right - recWin.left) - (recCli.right - recCli.left)), 48 + ((recWin.bottom - recWin.top) - (recCli.bottom - recCli.top)), SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOREDRAW | SWP_NOZORDER);

			/* Set the icon background colour */
			DrawCustomIconBack(hWnd_picMIconOpt);

			/* Set the correct item notify method option button */
			switch (lNotifyMethodOpt)
			{
				case NFY_VISUAL:
					CheckRadioButton(hDlg, IDC_btnNotify1, IDC_btnNotify2, IDC_btnNotify1);
					break;
				case NFY_SOUND:
					CheckRadioButton(hDlg, IDC_btnNotify1, IDC_btnNotify2, IDC_btnNotify2);
					break;
			}

			/* Set the item notify "Sound" play button bitmap */
			bmPlay = (HBITMAP)LoadImage(hInstOpt, MAKEINTRESOURCE(IDB_PLAY), IMAGE_BITMAP, 0, 0, 0);
			SendMessage(hWnd_cmdPlay, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)bmPlay);
			GetWindowRect(hWnd_picMIconOpt, &recWin);
			GetClientRect(hWnd_picMIconOpt, &recCli);
			SetWindowPos(hWnd_cmdPlay, NULL, 0, 0, 23 + ((recWin.right - recWin.left) - (recCli.right - recCli.left)), 19 + ((recWin.bottom - recWin.top) - (recCli.bottom - recCli.top)), SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOREDRAW | SWP_NOZORDER);

			/* Set the item notify "Sound" filename */
			GetClientRect(hWnd_cmdSoundFile, &recCli);
			SendMessage(hWnd_cmdSoundFile, WM_SETTEXT, 0, (LPARAM)CompactFilename(hWnd_cmdSoundFile, strSoundFileOpt, &recCli));

			return true;

		/* Paints the window */
		case WM_PAINT:
			hDCDlg = BeginPaint(hDlg, &psDlg);

			/* Draw custom icon background */
			DrawCustomIconBack(hWnd_picMIconOpt);

			EndPaint(hDlg, &psDlg);
  			return false;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_btnIcon1:
					if (HIWORD(wParam) == BN_CLICKED)
						bIconCustomBackOpt = false;
					return true;
				case IDC_btnIcon2:
					if (HIWORD(wParam) == BN_CLICKED)
						bIconCustomBackOpt = true;
					return true;

				case IDC_picMIconOpt:
					/* Build the ChooseColor structure */
					ccIconCustom.lStructSize = sizeof(CHOOSECOLOR);
					ccIconCustom.hwndOwner = hDlg;
					ccIconCustom.hInstance = NULL;
					ccIconCustom.rgbResult = cIconCustomBackOpt;
					ccIconCustom.lpCustColors = a_cCustomColors;
					ccIconCustom.Flags = CC_RGBINIT;
					ccIconCustom.lCustData = NULL;
					ccIconCustom.lpfnHook = NULL;
					ccIconCustom.lpTemplateName = NULL;
					
					/* Use ChooseColor common dialog to get color */
					if (ChooseColor(&ccIconCustom))
					{
						cIconCustomBackOpt = ccIconCustom.rgbResult;
						DrawCustomIconBack(GetDlgItem(hDlg, IDC_picMIconOpt));
					}
					
					return true;

				case IDC_btnNotify1:
					if (HIWORD(wParam) == BN_CLICKED)
						lNotifyMethodOpt = NFY_VISUAL;
					return true;
				case IDC_btnNotify2:
					if (HIWORD(wParam) == BN_CLICKED)
						lNotifyMethodOpt = NFY_SOUND;
					return true;
				
				case IDC_cmdSoundFile:
					/* Convert tokens to full path */
					DeTokenizeFilename(strFile, strSoundFileOpt);

					/* Generate initial path */
					strPos = strrchr(strFile, 92);	// Find last backslash
					if (strPos)
					{
						strncpy(strInitialDir, strFile, (strPos - strFile));
						strInitialDir[strPos - strFile] = 0;
					}
					else
						strInitialDir[0] = 0;

					/* Remove path from initial filename */
					strPos++;
					strcpy(strFile, strPos);	// NOTE: Same-string shuffle

					/* Find index to extension */
					strPos = strrchr(strFile, 46);	// Last period "."
					if (strPos)
					{
						strPos++;
						iFileExtIndex = (strPos - strFile);
					}
					else
						iFileExtIndex = 0;

					/* Build the OpenFileName structure */
					ofnSoundFile.lStructSize = sizeof(OPENFILENAME);
					ofnSoundFile.hwndOwner = hDlg;
					ofnSoundFile.hInstance = NULL;
					ofnSoundFile.lpstrFilter = "Sounds (*.wav)\0*.WAV\0All files (*.*)\0*.*\0";
					ofnSoundFile.lpstrCustomFilter = NULL;
					ofnSoundFile.nMaxCustFilter = 0;
					ofnSoundFile.nFilterIndex = 0;
					ofnSoundFile.lpstrFile = strFile;
					ofnSoundFile.nMaxFile = sizeof(strFile);
					ofnSoundFile.lpstrFileTitle = NULL;
					ofnSoundFile.nMaxFileTitle = 0;
					ofnSoundFile.lpstrInitialDir = strInitialDir;
					ofnSoundFile.lpstrTitle = NULL;
					ofnSoundFile.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_NOCHANGEDIR | OFN_HIDEREADONLY;
					ofnSoundFile.nFileOffset = 0;
					ofnSoundFile.nFileExtension = iFileExtIndex;
					ofnSoundFile.lpstrDefExt = "WAV";
					ofnSoundFile.lCustData = NULL;
					ofnSoundFile.lpfnHook = NULL;
					ofnSoundFile.lpTemplateName = NULL;

					/* Use GetOpenFileName common dialog to get filename */
					if (GetOpenFileName(&ofnSoundFile))
					{
						/* Convert full path to use tokens, if appropriate */
						TokenizeFilename(strSoundFileOpt, strFile);

						/* Update filename on the command button */
						GetClientRect(hWnd_cmdSoundFile, &recCli);
						SendMessage(hWnd_cmdSoundFile, WM_SETTEXT, 0, (LPARAM)CompactFilename(hWnd_cmdSoundFile, strSoundFileOpt, &recCli));
					}
					
					return true;

				case IDC_cmdPlay:
					/* Convert tokens to full path */
					DeTokenizeFilename(strFile, strSoundFileOpt);

					/* Play the sound */
					PlaySound(strFile, NULL, SND_FILENAME | SND_NODEFAULT | SND_ASYNC);
					
					return true;

				case IDC_cmdOptOK:
					EndDialog(hDlg, true);
					return true;
				case IDC_cmdOptCancel:
					EndDialog(hDlg, false);
					return true;
			}
			break;

		/* Window destroyed */
		case WM_DESTROY:
			/* Free the bitmap used by play button */
			DeleteObject(bmPlay);

			return false;
	}

	return false;
}

/* Compact filename for display */
char *CompactFilename(HWND hWndObject, char *strFilename, RECT *recDisplay)
{
	HDC hDCObject;
	HDC hDCCompact;
	static char strCompactFile[MAX_PATH];

	/* Create device context */
	hDCObject = GetDC(hWndObject);
	hDCCompact = CreateCompatibleDC(hDCObject);

	/* Copy filename to workarea */
	strcpy(strCompactFile, strFilename);

	/* Use DrawText API call to create the compacted filename */
	recDisplay->right = recDisplay->right + 75;
	DrawText(hDCCompact, strCompactFile, -1, recDisplay, DT_PATH_ELLIPSIS | DT_SINGLELINE | DT_MODIFYSTRING);
	
	/* Delete device context */
	DeleteDC(hDCCompact);
	ReleaseDC(hWndObject, hDCObject);

	return (strCompactFile);
}

/* Draw the custom icon background colour */
void DrawCustomIconBack(HWND hWndBack)
{
	HBRUSH brBackground;
	HBITMAP bmIcon;
	HDC hDCBack;
	HDC hDCMem;

	/* Get device context for icon */
	hDCBack = GetDC(hWndBack);

	/* Create coloured background */
	hDCMem = CreateCompatibleDC(hDCBack);
	bmIcon = (HBITMAP)LoadImage(hInstOpt, MAKEINTRESOURCE(IDB_DUMMYICON), IMAGE_BITMAP, 0, 0, LR_SHARED);
	bmIcon = (HBITMAP)SelectObject(hDCMem, bmIcon);
	brBackground = CreateSolidBrush(cIconCustomBackOpt);
	brBackground = (HBRUSH)SelectObject(hDCMem, brBackground);
	ExtFloodFill(hDCMem, 0, 0, cTransparentColor, FLOODFILLSURFACE);
	BitBlt(hDCBack, 0, 0, 48, 48, hDCMem, 0, 0, SRCCOPY);
	DeleteObject(SelectObject(hDCMem, bmIcon));
	DeleteObject(SelectObject(hDCMem, brBackground));
	DeleteDC(hDCMem);

	/* Release icon's device context */
	ReleaseDC(hWndBack, hDCBack);
}

