#include <windows.h>
#include <malloc.h>
#include <richedit.h>
#include "resource.h"
#include "Main.h"
#include "WatchGUI.h"

/* Function Declarations for internal use */
LRESULT CALLBACK WatchGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

/* Variables */
unsigned long lWatchItemsLengthOpt;
HINSTANCE hInstWatch;
char *strWatchItemsOpt;

/* The main routine */
BOOL WatchGUI(HINSTANCE hInstance, HWND hWndOwner)
{
	/* Save options to local variables */
	hInstWatch = hInstance;
	lWatchItemsLengthOpt = lWatchItemsLength;
	strWatchItemsOpt = (char *)malloc(lWatchItemsLengthOpt);
	memcpy(strWatchItemsOpt, strWatchItems, lWatchItemsLengthOpt);

	/* Show the options dialog */
	if (!DialogBox(hInstance, MAKEINTRESOURCE(IDD_WATCHGUI), hWndOwner, (DLGPROC)WatchGUIProc))
	{
		free(strWatchItemsOpt);
		return false;
	}

	/* User clicked OK, return updated options */
	lWatchItemsLength = lWatchItemsLengthOpt;
	free(strWatchItems);
	strWatchItems = strWatchItemsOpt;
	return true;
}

/* Window message handler */
LRESULT CALLBACK WatchGUIProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	unsigned long lTextLen;
	unsigned long lLoop;
	unsigned long lItemCnt;
	unsigned long lCharCnt;
	HWND hWnd_txtWatch;
	BOOL bAlpha;
	char *strItem;
	char *strText;
	GETTEXTLENGTHEX gtLength;
	GETTEXTEX gtText;

	/* Init variables */
	hWnd_txtWatch = GetDlgItem(hDlg, IDC_txtWatch);

	/* Process message */
	switch (uMsg)
	{
		/* Initialisation of dialog and controls */
		case WM_INITDIALOG:
			/* Build watch items text box */
			for (strItem = strWatchItemsOpt; strItem[0] != 0; strItem += (strlen(strItem) + 1))
			{
				SendMessage(hWnd_txtWatch, EM_REPLACESEL, false, (LPARAM)strItem);
				SendMessage(hWnd_txtWatch, EM_REPLACESEL, false, (LPARAM)"\n");
			}

			return true;

		/* Process commands */
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_txtWatch:
					if (HIWORD(wParam) == EN_SETFOCUS)
					{
						SendMessage(hWnd_txtWatch, EM_SETSEL, 0, 0);
						return false;
					}
					break;
				case IDC_cmdWatchOK:
					EndDialog(hDlg, true);
					return true;
				case IDC_cmdWatchCancel:
					EndDialog(hDlg, false);
					return true;
			}
			break;

		/* Window destroyed */
		case WM_DESTROY:
			/* Get length of buffer required to hold text */
			gtLength.flags = GTL_PRECISE | GTL_NUMBYTES;
			gtLength.codepage = CP_ACP;
			lTextLen = SendMessage(hWnd_txtWatch, EM_GETTEXTLENGTHEX, (WPARAM)&gtLength, 0);

			if (!lTextLen)
			{
				/* No text, init to zero */
				strText = NULL;
				lItemCnt = 0;
				lCharCnt = 0;
			}
			else
			{
				/* Get text from the watch items text box */
				gtText.cb = lTextLen;
				gtText.flags = GT_DEFAULT;
				gtText.codepage = CP_ACP;
				gtText.lpDefaultChar = NULL;
				gtText.lpUsedDefChar = NULL;
				strText = (char *)malloc(lTextLen);
				SendMessage(hWnd_txtWatch, EM_GETTEXTEX, (WPARAM)&gtText, (LPARAM)strText);

				/* Count number of watch items, and characters used */
				lItemCnt = 0;
				lCharCnt = 0;
				bAlpha = false;
				for (lLoop = 0; lLoop < lTextLen; lLoop++)
				{
					if ((strText[lLoop] == 10) || (strText[lLoop] == 13))
						bAlpha = false;
					else
					{
						if (!bAlpha)
						{
							bAlpha = true;
							lItemCnt++;
						}
						lCharCnt++;
					}
				}
			}

			/* Release previous watch item multi-string */
			free(strWatchItemsOpt);

			if (!lItemCnt)
			{
				/* No items, create empty multi-string */
				lWatchItemsLengthOpt = 2;
				strWatchItemsOpt = (char *)malloc(lWatchItemsLengthOpt);
				strWatchItemsOpt[0] = 0;
				strWatchItemsOpt[1] = 0;
			}
			else
			{
				/* Allocate space for new watch item multi-string */
				lWatchItemsLengthOpt = lCharCnt + lItemCnt + 1;
				strWatchItemsOpt = (char *)malloc(lWatchItemsLengthOpt);

				/* Build the new watch item multi-string */
				strItem = strWatchItemsOpt;
				bAlpha = false;
				for (lLoop = 0; lLoop < lTextLen; lLoop++)
				{
					if ((strText[lLoop] == 10) || (strText[lLoop] == 13))
					{
						if (bAlpha)
						{
							bAlpha = false;
							*strItem = 0;
							strItem++;
						}
					}
					else
					{
						if (!bAlpha)
							bAlpha = true;
						*strItem = tolower(strText[lLoop]);
						strItem++;
					}
				}
				if (bAlpha)
				{
					*strItem = 0;
					strItem++;
				}
				*strItem = 0;
			}
			
			/* Release text work area */
			free(strText);

			return false;
	}

	return false;
}
